#include "stdafx.h"

#include "Fragment.h"
#include "TextureTools.h"


/*static*/ void CFragment::vAddFragment(CFragment* lpLstFragments,DWORD& dwNumFragments,TR2_OBJECT_TEXTURE* lpObjVert,BYTE bNumVertexInObjText, enumOBJTEXTType objTextType,void* lpUserData,bool boDontRegroup,BYTE* lpbTextureData,DWORD dwTextureWidth)
{
    ASSERT(lpbTextureData != NULL);

    bool        boQuad      = (bNumVertexInObjText == 4);

    // Look for a fragment which would already be / contain the one
    // implicated by lpObjVert
    DWORD   dwFragmentFound = -1;

    if(!boDontRegroup)
        {
        // Scan the fragments already built
        for(DWORD dwK = 0; dwK < dwNumFragments; dwK++)
            {
            CFragment* lpCurFragment = &lpLstFragments[dwK];

            // Look only for fragments which are in the same tile
            if(lpbTextureData == lpCurFragment->m_lpbTextureData)
                {
                // Compute the extents of lpObjVert
                bit16   sMinU,sMinV,sMaxU,sMaxV;

                CTRFile::vComputeObjTextureExtents(lpObjVert,bNumVertexInObjText,true,&sMinU,&sMinV,&sMaxU,&sMaxV);

                // No need to create a new fragment if the current one is/contain
                // the one implicated by lpObjVert
                bool    boOk =  (lpCurFragment->m_sUMin <= sMinU)  &&
                                (lpCurFragment->m_sVMin <= sMinV)  &&
                                (lpCurFragment->m_sUMax >= sMaxU)  &&
                                (lpCurFragment->m_sVMax >= sMaxV);
            
                if(boOk)
                    {
                    dwFragmentFound = dwK;
                    break;
                    }
                }
            }
        }

    // Fragment not found ? Create a new one
    if(dwFragmentFound == -1)
        {
        bit16   sMinU,sMinV,sMaxU,sMaxV;

        CTRFile::vComputeObjTextureExtents(lpObjVert,bNumVertexInObjText,false,&sMinU,&sMinV,&sMaxU,&sMaxV);

        lpLstFragments[dwNumFragments].m_lpbTextureData = lpbTextureData;
        lpLstFragments[dwNumFragments].m_dwTextureWidth = dwTextureWidth;
        lpLstFragments[dwNumFragments].m_sUMin          = sMinU;
        lpLstFragments[dwNumFragments].m_sVMin          = sMinV;
        lpLstFragments[dwNumFragments].m_sUMax          = sMaxU;
        lpLstFragments[dwNumFragments].m_sVMax          = sMaxV;
        lpLstFragments[dwNumFragments].m_sWidth         = sMaxU - sMinU + 1;
        lpLstFragments[dwNumFragments].m_sHeight        = sMaxV - sMinV + 1;
        lpLstFragments[dwNumFragments].m_dwArea         = ((DWORD) lpLstFragments[dwNumFragments].m_sWidth) * lpLstFragments[dwNumFragments].m_sHeight;

        dwFragmentFound = dwNumFragments++;
        }

    // Add a new texture fragment to the fragment
    FRAGMENT_TEXTURE    FragText;

    FragText.ObjText        = *lpObjVert;
    FragText.boQuad         = boQuad;
    FragText.lpUserData     = lpUserData;
    FragText.ObjTextType    = objTextType;
    FragText.boAlreadyDone  = false;

    lpLstFragments[dwFragmentFound].m_LstTextCoord.push_back(FragText);
}


/*static*/ void CFragment::vCollectFragments(CTRFile* lpTRFile, TR2_OBJECT_TEXTURE* lpOText, DWORD dwNumPolys, BYTE bNumVertexPerPoly,void* lpLstPolys,CFragment* lpLstFragments,DWORD& dwNumFragments)
{
    //
    // Collect fragments for a list of faces (tris or quads)
    //

    TR_FILE&    trFile          = lpTRFile->GetTRFile();
    TR2_FACE3*  lpLstTris       = (TR2_FACE3*) lpLstPolys;
    TR2_FACE4*  lpLstQuads      = (TR2_FACE4*) lpLstPolys;
    bool        boQuad          = (bNumVertexPerPoly == 4);

    for(bit16 sI = 0; sI < dwNumPolys; sI++, lpLstTris++, lpLstQuads++)
        {
        TR2_OBJECT_TEXTURE* lpObjVert = &lpOText[boQuad ? lpLstQuads->wTextureIndex & 0x7FFF : lpLstTris->wTextureIndex & 0x7FFF];

        BYTE*   lpbTextureData = (BYTE*) trFile.tr123File.lplpTextiles16[lpObjVert->wTile]->awTile;

        vAddFragment(lpLstFragments,dwNumFragments,lpObjVert,bNumVertexPerPoly,OBJTEXT_TYPE_POLY,boQuad ? (void*)lpLstQuads : (void*)lpLstTris, false, lpbTextureData);
        }
}


/*static*/ void CFragment::vCollectFragments(BYTE* lpbTextureData, DWORD dwTextureWidth, TR2_OBJECT_TEXTURE* lpOText, DWORD dwNumPolys, BYTE bNumVertexPerPoly,void* lpLstPolys,CFragment* lpLstFragments,DWORD& dwNumFragments)
{
    //
    // Collect fragments for a list of faces (tris or quads)
    //

    TR2_FACE3*  lpLstTris       = (TR2_FACE3*) lpLstPolys;
    TR2_FACE4*  lpLstQuads      = (TR2_FACE4*) lpLstPolys;
    bool        boQuad          = (bNumVertexPerPoly == 4);

    for(bit16 sI = 0; sI < dwNumPolys; sI++, lpLstTris++, lpLstQuads++)
        {
        TR2_OBJECT_TEXTURE* lpObjVert = &lpOText[boQuad ? lpLstQuads->wTextureIndex & 0x7FFF : lpLstTris->wTextureIndex & 0x7FFF];

        vAddFragment(lpLstFragments,dwNumFragments,lpObjVert,bNumVertexPerPoly,OBJTEXT_TYPE_POLY,boQuad ? (void*)lpLstQuads : (void*)lpLstTris, false, lpbTextureData,dwTextureWidth);
        }
}


/*static*/ void CFragment::vCollectFragments(CTRFile* lpTRFile, TR123_MESH* lpMesh,CFragment** lplpLstFragments,DWORD* lpdwNumFragments,DWORD dwNumMeshes)
{
    //
    // Collect fragments for one or several meshes
    //

    DWORD   dwNumMaxFragments = 0;

    for(DWORD dwI = 0; dwI < dwNumMeshes; dwI++)
        {
        dwNumMaxFragments += lpMesh[dwI].sNumTexturedRectangles + lpMesh[dwI].sNumTexturedTriangles;
        }

    DWORD   dwNumFragments      = 0;

    *lplpLstFragments = new CFragment [ dwNumMaxFragments ];

    while(dwNumMeshes--)
        {
        if(lpMesh[dwNumMeshes].boIsDummy) continue;

        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpMesh[dwNumMeshes].sNumTexturedRectangles, 4, lpMesh[dwNumMeshes].lpTexturedRectangles, *lplpLstFragments, dwNumFragments);
        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpMesh[dwNumMeshes].sNumTexturedTriangles,  3, lpMesh[dwNumMeshes].lpTexturedTriangles,  *lplpLstFragments, dwNumFragments);
        }

    *lpdwNumFragments = dwNumFragments;
}


/*static*/ void CFragment::vCollectFragments(CTrLstMsh* lpMsh,CFragment* lpLstFragments,DWORD& dwNumFragments)
{
    //
    // Collect fragments from a CTrLstMsh
    //

    TR2_OBJECT_TEXTURE* lpOText = const_cast<TR2_OBJECT_TEXTURE*>(lpMsh->lpGetFragText());

    DWORD   dwNumMeshes = lpMsh->dwGetNumMeshes();

    while(dwNumMeshes--)
        {
        const TR123_MESH* lpMesh = lpMsh->lpGetMesh(dwNumMeshes);

        if(lpMesh->boIsDummy) continue;

        vCollectFragments(const_cast<BYTE*>(lpMsh->lpGetTextureData()), lpMsh->dwGetTextureWidth(), lpOText, lpMesh->sNumTexturedRectangles, 4, lpMesh->lpTexturedRectangles, lpLstFragments, dwNumFragments);
        vCollectFragments(const_cast<BYTE*>(lpMsh->lpGetTextureData()), lpMsh->dwGetTextureWidth(), lpOText, lpMesh->sNumTexturedTriangles,  3, lpMesh->lpTexturedTriangles,  lpLstFragments, dwNumFragments);
        }
}


/*static*/ void CFragment::vCollectFragments(CTRFile* lpTRFile, CFragment** lplpLstFragments,DWORD* lpdwNumFragments)
{
    //
    // Collect fragments for the whole level
    //

    // Calculate an approximated num max of fragments
    TR_FILE& trFile = lpTRFile->GetTRFile();

    DWORD       dwNumMaxFragments = 0;
    TR123_MESH* lpMesh;

    dwNumMaxFragments  = trFile.tr123File.dwNumObjectTextures;
    dwNumMaxFragments += trFile.tr123File.dwNumSpriteTextures;

    // Collect the fragments...
    DWORD   dwNumFragments = 0;

    *lplpLstFragments = new CFragment [ dwNumMaxFragments ];

    // ... from the meshes
    for(DWORD dwI = 0; dwI < trFile.tr123File.dwNumMeshes; dwI++)
        {
        lpMesh = trFile.tr123File.lplpMeshes[dwI];
        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpMesh->sNumTexturedRectangles, 4, lpMesh->lpTexturedRectangles, *lplpLstFragments, dwNumFragments);
        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpMesh->sNumTexturedTriangles,  3, lpMesh->lpTexturedTriangles,  *lplpLstFragments, dwNumFragments);
        }

    // ... from the sprites
    for(dwI = 0; dwI < trFile.tr123File.dwNumSpriteTextures; dwI++)
        {
        TR2_SPRITE_TEXTURE* lpSprText = &trFile.tr123File.lpSpriteTextures[dwI];

        WORD wTile      = lpSprText->wTile;
        BYTE bX         = lpSprText->bX;
        BYTE bY         = lpSprText->bY;
        WORD wWidth     = (lpSprText->wWidth - 255) / 256;
        WORD wHeight    = (lpSprText->wHeight - 255) / 256;

        BYTE*   lpbTextureData = (BYTE*) trFile.tr123File.lplpTextiles16[wTile]->awTile;

        TR2_OBJECT_TEXTURE  ObjText;

        ObjText.wTile       = wTile;
        ObjText.wAttribute  = 0;
        ObjText.Vertices[0].bXPixel = bX;           ObjText.Vertices[0].cXCoordinate = 0;
        ObjText.Vertices[0].bYPixel = bY;           ObjText.Vertices[0].cYCoordinate = 0;
        ObjText.Vertices[1].bXPixel = bX + wWidth;  ObjText.Vertices[1].cXCoordinate = 0;
        ObjText.Vertices[1].bYPixel = bY;           ObjText.Vertices[1].cYCoordinate = 0;
        ObjText.Vertices[2].bXPixel = bX + wWidth;  ObjText.Vertices[2].cXCoordinate = 0;
        ObjText.Vertices[2].bYPixel = bY + wHeight; ObjText.Vertices[2].cYCoordinate = 0;
        ObjText.Vertices[3].bXPixel = bX;           ObjText.Vertices[3].cXCoordinate = 0;
        ObjText.Vertices[3].bYPixel = bY + wHeight; ObjText.Vertices[3].cYCoordinate = 0;

        vAddFragment(*lplpLstFragments,dwNumFragments,&ObjText, 4, OBJTEXT_TYPE_SPRITE, (void*)lpSprText, false, lpbTextureData);
        }

    // ... from the animated textures
    bitu16* lpsAnimatedTextures     = trFile.tr123File.lpAnimatedTextures;
    WORD    wNumAnimatedTextures    = *lpsAnimatedTextures++;

    while(wNumAnimatedTextures--)
        {
        bitu16   wNum = *lpsAnimatedTextures++;  wNum++;
        while(wNum--)
            {
            bitu16  wNumObjText = *lpsAnimatedTextures;
            TR2_OBJECT_TEXTURE* lpObjVert = &trFile.tr123File.lpObjectTextures[wNumObjText];
            bool    boTri = (lpObjVert->Vertices[3].bXPixel == 0)  &&
                            (lpObjVert->Vertices[3].bYPixel == 0)  &&
                            (lpObjVert->Vertices[3].cXCoordinate == 0)  &&
                            (lpObjVert->Vertices[3].cYCoordinate == 0);

            BYTE*   lpbTextureData = (BYTE*) trFile.tr123File.lplpTextiles16[lpObjVert->wTile]->awTile;

            vAddFragment(*lplpLstFragments,dwNumFragments,lpObjVert, boTri ? 3 : 4, OBJTEXT_TYPE_ANIMATEDTEXTURE,(void*)lpsAnimatedTextures, false, lpbTextureData);
            lpsAnimatedTextures++;
            }
        }

    // ... from the rooms
    for(dwI = 0; dwI < trFile.tr123File.wNumRooms; dwI++)
        {
        TR123_ROOM*     lpRoom = &trFile.tr123File.lpRooms[dwI];
        TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpRoomData->wNumRectangles, 4, lpRoomData->lpRectangles, *lplpLstFragments, dwNumFragments);
        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpRoomData->wNumTriangles,  3, lpRoomData->lpTriangles,  *lplpLstFragments, dwNumFragments);
        }

    *lpdwNumFragments = dwNumFragments;
}


/*static*/ void CFragment::vCollectFragmentsV2(CTRFile* lpTRFile, CFragment** lplpLstFragments,DWORD* lpdwNumFragments)
{
    //
    // Collect fragments for the whole level
    //

    // Calculate an approximated num max of fragments
    TR_FILE& trFile = lpTRFile->GetTRFile();

    DWORD       dwNumMaxFragments = 0;

    TRListStaticMesh*   lpListStatMsh   = trFile.tr123File.lpListStaticMesh;
    TRListMoveable*     lpListMvb       = trFile.tr123File.lpListMoveable;
    TRListSpriteSeq*    lpListSpriteSeq = trFile.tr123File.lpListSpriteSeq;

    for(TRListMoveable::iterator theIterator = lpListMvb->begin(); theIterator != lpListMvb->end(); theIterator++)
        {
        CTrMvb* lpMvb = *theIterator;

        dwNumMaxFragments += lpMvb->dwGetNumFragments();
        }

    for(TRListStaticMesh::iterator theIterator2 = lpListStatMsh->begin(); theIterator2 != lpListStatMsh->end(); theIterator2++)
        {
        CTrStatMsh* lpStatMsh = *theIterator2;

        dwNumMaxFragments += lpStatMsh->dwGetNumFragments();
        }

    for(TRListSpriteSeq::iterator theIterator3 = lpListSpriteSeq->begin(); theIterator3 != lpListSpriteSeq->end(); theIterator3++)
        {
        CTrLstSpr* lpSpr = *theIterator3;

        dwNumMaxFragments += lpSpr->dwGetNumFragments();
        }

    dwNumMaxFragments += (*trFile.tr123File.lpAnimatedTextures);

    for(DWORD dwNumRoom = 0; dwNumRoom < trFile.tr123File.wNumRooms; dwNumRoom++)
        {
        TR123_ROOM*     lpRoom = &trFile.tr123File.lpRooms[dwNumRoom];
        TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

        dwNumMaxFragments += lpRoomData->wNumRectangles + lpRoomData->wNumTriangles;
        }

    // Collect the fragments...
    DWORD   dwNumFragments = 0;

    *lplpLstFragments = new CFragment [ dwNumMaxFragments ];

    // ... from the moveables
    for(theIterator = lpListMvb->begin(); theIterator != lpListMvb->end(); theIterator++)
        {
        CTrMvb* lpMvb = *theIterator;

        vCollectFragments(lpMvb,*lplpLstFragments,dwNumFragments);
        }

    // ... from the static meshes
    for(theIterator2 = lpListStatMsh->begin(); theIterator2 != lpListStatMsh->end(); theIterator2++)
        {
        CTrStatMsh* lpStatMsh = *theIterator2;

        vCollectFragments(lpStatMsh,*lplpLstFragments,dwNumFragments);
        }

    // ... from the sprites
    for(theIterator3 = lpListSpriteSeq->begin(); theIterator3 != lpListSpriteSeq->end(); theIterator3++)
        {
        CTrLstSpr* lpSpr = *theIterator3;

        TR2_OBJECT_TEXTURE* lpOText = const_cast<TR2_OBJECT_TEXTURE*>(lpSpr->lpGetFragText());

        for(DWORD dwI = 0; dwI < lpSpr->dwGetNumSprites(); dwI++)
            {
            vAddFragment(*lplpLstFragments,dwNumFragments,lpOText, 4, OBJTEXT_TYPE_SPRITE2,(void*)lpOText, false, (BYTE*) lpSpr->lpGetTextureData(), lpSpr->dwGetTextureWidth());
            lpOText++;
            }
        }

    // ... from the animated textures
    bitu16* lpsAnimatedTextures     = trFile.tr123File.lpAnimatedTextures;
    WORD    wNumAnimatedTextures    = *lpsAnimatedTextures++;

    while(wNumAnimatedTextures--)
        {
        bitu16   wNum = *lpsAnimatedTextures++;  wNum++;
        while(wNum--)
            {
            bitu16  wNumObjText = *lpsAnimatedTextures;
            TR2_OBJECT_TEXTURE* lpObjVert = &trFile.tr123File.lpObjectTextures[wNumObjText];
            bool    boTri = (lpObjVert->Vertices[3].bXPixel == 0)  &&
                            (lpObjVert->Vertices[3].bYPixel == 0)  &&
                            (lpObjVert->Vertices[3].cXCoordinate == 0)  &&
                            (lpObjVert->Vertices[3].cYCoordinate == 0);

            BYTE*   lpbTextureData = (BYTE*) trFile.tr123File.lplpTextiles16[lpObjVert->wTile]->awTile;

            vAddFragment(*lplpLstFragments,dwNumFragments,lpObjVert, boTri ? 3 : 4, OBJTEXT_TYPE_ANIMATEDTEXTURE,(void*)lpsAnimatedTextures, false, lpbTextureData);
            lpsAnimatedTextures++;
            }
        }

    // ... from the rooms
    for(DWORD dwI = 0; dwI < trFile.tr123File.wNumRooms; dwI++)
        {
        TR123_ROOM*     lpRoom = &trFile.tr123File.lpRooms[dwI];
        TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpRoomData->wNumRectangles, 4, lpRoomData->lpRectangles, *lplpLstFragments, dwNumFragments);
        vCollectFragments(lpTRFile, lpTRFile->GetTRFile().tr123File.lpObjectTextures, lpRoomData->wNumTriangles,  3, lpRoomData->lpTriangles,  *lplpLstFragments, dwNumFragments);
        }

    *lpdwNumFragments = dwNumFragments;
}


/*static*/ void CFragment::vSortFragments(CFragment* lpLstFragments,int dwIdxLeft,int dwIdxRight)
{
    //
    // Sort the fragments. Fragments are first sorted by height then by width
    // (in descending order for both keys)
    //

    int     dwIdx   = dwIdxLeft;
    int     dwJdx   = dwIdxRight;
    int     dwIndex = (dwIdxLeft + dwIdxRight) / 2;
    int     dwVal1  = lpLstFragments[dwIndex].m_sHeight;
    int     dwVal2  = lpLstFragments[dwIndex].m_sWidth;

    while(dwIdx <= dwJdx)
        {
        while(((lpLstFragments[dwIdx].m_sHeight > dwVal1)  ||
              (lpLstFragments[dwIdx].m_sHeight == dwVal1  &&  lpLstFragments[dwIdx].m_sWidth > dwVal2)) &&
              (dwIdx < dwIdxRight))
              dwIdx++;

        while(((dwVal1 > lpLstFragments[dwJdx].m_sHeight)  ||
              (lpLstFragments[dwJdx].m_sHeight == dwVal1  &&  lpLstFragments[dwJdx].m_sWidth < dwVal2)) &&
              (dwJdx > dwIdxLeft))
              dwJdx--;

        if(dwIdx <= dwJdx)
            {
            CFragment temp = lpLstFragments[dwIdx];
            lpLstFragments[dwIdx] = lpLstFragments[dwJdx];
            lpLstFragments[dwJdx] = temp;
            dwIdx++;
            dwJdx--;
            }
        }

    if(dwIdxLeft < dwJdx)   vSortFragments(lpLstFragments,dwIdxLeft,dwJdx);
    if(dwIdx < dwIdxRight)  vSortFragments(lpLstFragments,dwIdx,dwIdxRight);
}


/*static*/ DWORD CFragment::dwRemoveDuplicated(DWORD dwNumFragments,CFragment* lpLstFragments)
{
    ASSERT(lpLstFragments);

    if(dwNumFragments < 2) return 0;

    DWORD   dwMemory = 0;

    // Scan all the fragments
    for(DWORD dwFragment = 0; dwFragment < dwNumFragments - 1; dwFragment++)
        {
        CFragment* lpFrag1 = &lpLstFragments[dwFragment];

        // Look for fragments with the same height / width
        for(DWORD dwFragment2 = dwFragment + 1; dwFragment2 < dwNumFragments; dwFragment2++)
            {
            CFragment* lpFrag2 = &lpLstFragments[dwFragment2];

            if(lpFrag1->m_sWidth != lpFrag2->m_sWidth  ||  lpFrag1->m_sHeight != lpFrag2->m_sHeight) break;
            if(lpFrag2->m_LstTextCoord.size() == 0) continue;

            // Compare both fragments
            bool boOk = true;

            for(DWORD dwJ = 0; dwJ < lpFrag1->m_sHeight; dwJ++)
                {
                BYTE*   lpData1 = lpFrag1->m_lpbTextureData + (lpFrag1->m_sVMin + dwJ) * lpFrag1->m_dwTextureWidth * 2 + lpFrag1->m_sUMin * 2;
                BYTE*   lpData2 = lpFrag2->m_lpbTextureData + (lpFrag2->m_sVMin + dwJ) * lpFrag2->m_dwTextureWidth * 2 + lpFrag2->m_sUMin * 2;

                if(memcmp(lpData1, lpData2, lpFrag1->m_sWidth * 2) != 0)
                    {
                    boOk = false;
                    break;
                    }
                }

            if(!boOk) continue;

            // Fragment 1 & 2 are equal, so add the OBJECT_TEXTURE of the second
            // to the first one
            for(LISTOBJTEXT::iterator theIter = lpFrag2->m_LstTextCoord.begin(); theIter != lpFrag2->m_LstTextCoord.end(); theIter++)
                {
                FRAGMENT_TEXTURE*   lpFragText = &(*theIter);
                BYTE                bNumVert = lpFragText->boQuad ? 4 : 3;

                bit16 sTransX = lpFrag1->m_sUMin - lpFrag2->m_sUMin;
                bit16 sTransY = lpFrag1->m_sVMin - lpFrag2->m_sVMin;

                for(BYTE bVert = 0; bVert < bNumVert; bVert++)
                    {
                    lpFragText->ObjText.Vertices[bVert].bXPixel += sTransX;
                    lpFragText->ObjText.Vertices[bVert].bYPixel += sTransY;
                    }

                lpFrag1->m_LstTextCoord.push_back(*lpFragText);
                }

            lpFrag2->m_LstTextCoord.clear();

            dwMemory += lpFrag1->m_sHeight * lpFrag1->m_sWidth * 2;
            }
        }

    return dwMemory;
}


/*static*/ void CFragment::vOrganizeFragments(CTRFile* lpTRFile,DWORD dwNumFragments,CFragment* lpLstFragments,
    BYTE* lpbArea,DWORD dwAreaWidth,bool boCalculateOnly,DWORD& dwRemHeight,DWORD& dwWidthMax,bool boAlign256)
{
    //
    // Organize fragments into the texture area. The algo is simple: we fill the area by "bands".
    // The height of the band is the height of the first fragment we put on the left of the band. Then we
    // fill the band with the other fragments, putting them under the previous fragments, and if not possible
    // (if adding the fragment would go beyond the height of the band), putting them to the right of the
    // previous one. When we reach the end of the band (width=256), we just begin a new band behind the
    // previous one. Because fragments are sorted by decrementing height, we know that when a fragment
    // is put into the texture area to begin a band, the next ones will always fit into the band (ie.
    // they will not go beyond the height of the band).
    //

    DWORD   dwRemWidthInBand    = 0;    // "Rem" in variable names stands for "remaining"
    DWORD   dwRemHeightInCell   = 0;
    DWORD   dwBandHeight        = 0;
    DWORD   dwCellWidth         = 0;
    DWORD   dwCurY = 0, dwCurX = 0;
    
    dwWidthMax = 0;

    BYTE*   lpbData = NULL;
    DWORD   dwTextureDepth = 16;

    // Go through the fragments
    for(DWORD dwI = 0; dwI < dwNumFragments; dwI++)
        {
        if(lpLstFragments[dwI].m_LstTextCoord.size() == 0) continue;

        DWORD   dwFragmentHeight = lpLstFragments[dwI].m_sHeight;
        DWORD   dwFragmentWidth  = lpLstFragments[dwI].m_sWidth;
        DWORD   dwX = 0, dwY = 0;
        
        // Get in lpbData the texture where this fragment lives
        if(!boCalculateOnly)
            {
            lpbData = lpLstFragments[dwI].m_lpbTextureData;
            }

        // Need to create a new band ?
        bool boNewBand = (dwRemWidthInBand < dwFragmentWidth)  ||
              ((dwRemHeightInCell < dwFragmentHeight)  &&  (dwRemWidthInBand - dwCellWidth < dwFragmentWidth));

        // Yes
        if(boNewBand)
            {
            if(dwI  &&  (dwAreaWidth - (dwRemWidthInBand - dwCellWidth) > dwWidthMax)) dwWidthMax = dwAreaWidth - dwRemWidthInBand + dwCellWidth;

            // Adjust the remaining height available in the texture area
            dwRemHeight -= dwBandHeight;
            if(dwRemHeight < dwFragmentHeight)
                {
                AfxMessageBox("Not enough room in the area !");
                exit(-1);
                }
            else
                {
                // Adjust the (x,y) position in the texture area for the next fragment
                // We begin a new band -> x = 0
                dwCurX = 0;
                dwCurY += dwRemHeightInCell;
                if(boAlign256)
                    {
                    if((dwCurY / 256) != ((dwCurY + dwFragmentHeight - 1) / 256))
                        {
                        DWORD dwHeightLost = ((dwCurY / 256) + 1) * 256 - dwCurY;
                        dwRemHeight -= dwHeightLost;
                        dwCurY += dwHeightLost;
                        }
                    }
                dwRemWidthInBand    = dwAreaWidth;
                dwBandHeight        = dwFragmentHeight;
                dwCellWidth         = dwFragmentWidth;
                dwRemHeightInCell   = 0;
                dwX = dwCurX;
                dwY = dwCurY; dwCurY += dwFragmentHeight;
                }
            }
        // No need to create a new band. Is it possible to put the fragment under the
        // previous one ?
        else if(dwRemHeightInCell < dwFragmentHeight)
            {
            // No, put the fragment to the right of the previous one
            dwCurX += dwCellWidth;
            dwCurY -= dwBandHeight - dwRemHeightInCell;
            dwRemWidthInBand -= dwCellWidth;
            dwCellWidth = dwFragmentWidth;
            dwRemHeightInCell = dwBandHeight - dwFragmentHeight;
            dwX = dwCurX;
            dwY = dwCurY; dwCurY += dwFragmentHeight;
            }
        else
            {
            // Yes, so just do it
            if(dwFragmentWidth > dwCellWidth) dwCellWidth = dwFragmentWidth;
            dwRemHeightInCell -= dwFragmentHeight;
            dwX = dwCurX;
            dwY = dwCurY; dwCurY += dwFragmentHeight;
            }

        // Copy the fragment data into the texture area
        if(!boCalculateOnly)
            {
            ASSERT(lpbData != NULL);
            for(DWORD dwH = 0; dwH < dwFragmentHeight; dwH++)
                {
                memcpy(&lpbArea[(dwX + (dwY + dwH) * dwAreaWidth) * (dwTextureDepth / 8)], &lpbData[(lpLstFragments[dwI].m_sUMin + (lpLstFragments[dwI].m_sVMin + dwH) * lpLstFragments[dwI].m_dwTextureWidth) * (dwTextureDepth / 8)], dwFragmentWidth * (dwTextureDepth / 8));
                }
            // Save the position where the fragment was copied
            lpLstFragments[dwI].m_sNewUMin = dwX;
            lpLstFragments[dwI].m_sNewVMin = dwY;
            }
        }

    dwRemHeight -= dwBandHeight;
    if(dwAreaWidth - (dwRemWidthInBand - dwCellWidth) > dwWidthMax) dwWidthMax = dwAreaWidth - dwRemWidthInBand + dwCellWidth;
}


/*static*/ DWORD CFragment::dwBuildTextureFromFragments(CTRFile* lpTRFile,CFragment* lpLstFragments,DWORD dwNumFragments,BYTE** lplpbArea,DWORD* lpdwAreaWidth,DWORD* lpdwAreaHeight,bool boAlign256)
{
    //
    // Given a fragments list, build a texture area with all the fragments in it and
    // recalculate all the TR2_OBJECT_TEXTURE structures to reflect the new ordering of the textures
    //

    DWORD   dwWidthMax;
    DWORD   dwAreaWidth     = *lpdwAreaWidth ? *lpdwAreaWidth : 256;
    DWORD   dwAreaHeight    = 100000;
    DWORD   dwTextureDepth  = 16;

    // Just calculate the texture height we need to contain all the ordered fragments
    vOrganizeFragments(lpTRFile,dwNumFragments,lpLstFragments,NULL,dwAreaWidth,true,dwAreaHeight,dwWidthMax,boAlign256);

    dwAreaWidth     = *lpdwAreaWidth ? *lpdwAreaWidth : dwWidthMax;
    dwAreaHeight    = *lpdwAreaHeight ? *lpdwAreaHeight : 100000 - dwAreaHeight;

    // Allocate the texture area
    DWORD dwAreaSize = dwAreaWidth * dwAreaHeight * (dwTextureDepth / 8);
    *lplpbArea = new BYTE [ dwAreaSize ];
    memset(*lplpbArea, 255, dwAreaSize);

    // Copy the fragments into the area
    DWORD dwHeight = dwAreaHeight;

    vOrganizeFragments(lpTRFile,dwNumFragments,lpLstFragments,*lplpbArea,dwAreaWidth,false,dwHeight,dwWidthMax,boAlign256);

    // Offset the TR2_OBJECT_TEXTURE coordinates according to where the fragments were copied
    for(DWORD dwI = 0; dwI < dwNumFragments; dwI++)
        {
        WORD    wTile   = lpLstFragments[dwI].m_sNewVMin / 256;
        bit16   sTransX = lpLstFragments[dwI].m_sNewUMin - lpLstFragments[dwI].m_sUMin;
        bit16   sTransY = (lpLstFragments[dwI].m_sNewVMin & 0xFF) - lpLstFragments[dwI].m_sVMin;
        
        for(LISTOBJTEXT::iterator theIter = lpLstFragments[dwI].m_LstTextCoord.begin(); theIter != lpLstFragments[dwI].m_LstTextCoord.end(); theIter++)
            {
            FRAGMENT_TEXTURE*   lpFragText = &(*theIter);
            TR2_OBJECT_TEXTURE* lpObjVert = &lpFragText->ObjText;

            lpObjVert->wTile = wTile;

            BYTE    bNumVert = lpFragText->boQuad ? 4 : 3;

            for(BYTE bJ = 0; bJ < bNumVert; bJ++)
                {
                bit16   sXPixel = lpObjVert->Vertices[bJ].bXPixel; sXPixel += sTransX;
                lpObjVert->Vertices[bJ].bXPixel = sXPixel;

                bit16   sYPixel = lpObjVert->Vertices[bJ].bYPixel; sYPixel += sTransY;
                lpObjVert->Vertices[bJ].bYPixel = sYPixel;
                }
            }
        }

    // Set the indexes to TR2_OBJECT_STRUCTURE of triangles / rectangles / animated textures
    // Set the new tile / (x,y) position for each sprite
    WORD wIndex = 0;
    for(DWORD dwN = 0; dwN < dwNumFragments; dwN++)
        {
        // For each fragment, scan all the fragments texture
        for(LISTOBJTEXT::iterator theIter = lpLstFragments[dwN].m_LstTextCoord.begin(); theIter != lpLstFragments[dwN].m_LstTextCoord.end(); theIter++)
            {
            FRAGMENT_TEXTURE* lpFragText = &(*theIter);

            // If this fragment texture is already processed, continue
            if(lpFragText->boAlreadyDone) continue;

            // Look for identical fragments texture in the list
            for(LISTOBJTEXT::iterator theIter2 = theIter; theIter2 != lpLstFragments[dwN].m_LstTextCoord.end(); theIter2++)
                {
                FRAGMENT_TEXTURE* lpFragText2 = &(*theIter2);

                if(lpFragText2->boAlreadyDone) continue;
                if(*lpFragText != *lpFragText2) continue;

                switch(lpFragText2->ObjTextType)
                    {
                    // If this is a fragment texture for a poly, adjust the TextureIndex
                    case OBJTEXT_TYPE_POLY:
                        {
                        if(lpFragText2->boQuad)
                            {
                            // Preserve the MSB which can be set for polys in TR3...
                            WORD    wMSB = ((TR2_FACE4*)lpFragText2->lpUserData)->wTextureIndex & 0x8000;
                            ((TR2_FACE4*)lpFragText2->lpUserData)->wTextureIndex = wIndex | wMSB;
                            }
                        else
                            {
                            // Preserve the MSB which can be set for polys in TR3...
                            WORD    wMSB = ((TR2_FACE3*)lpFragText2->lpUserData)->wTextureIndex & 0x8000;
                            ((TR2_FACE3*)lpFragText2->lpUserData)->wTextureIndex = wIndex | wMSB;
                            }
                        break;
                        }
                    // If this is a fragment texture for a sprite, adjust the position / tile
                    case OBJTEXT_TYPE_SPRITE:
                        {
                        TR2_SPRITE_TEXTURE* lpSprText = (TR2_SPRITE_TEXTURE*) lpFragText2->lpUserData;

                        lpSprText->wTile = lpFragText2->ObjText.wTile;
                        lpSprText->bX = lpFragText2->ObjText.Vertices[0].bXPixel;
                        lpSprText->bY = lpFragText2->ObjText.Vertices[0].bYPixel;
                        break;
                        }
                    case OBJTEXT_TYPE_SPRITE2:
                        {
                        TR2_OBJECT_TEXTURE* lpOText = (TR2_OBJECT_TEXTURE*) lpFragText2->lpUserData;
                        
                        long    lTransX = (long)lpFragText2->ObjText.Vertices[0].bXPixel - (long)lpOText->Vertices[0].bXPixel;
                        long    lTransY = (long)lpFragText2->ObjText.Vertices[0].bYPixel - (long)lpOText->Vertices[0].bYPixel;

                        for(BYTE bJ = 0; bJ < 4; bJ++)
                            {
                            lpOText->Vertices[bJ].bXPixel += lTransX;
                            lpOText->Vertices[bJ].bYPixel += lTransY;
                            }

                        lpOText->wTile = lpFragText2->ObjText.wTile;
                        break;
                        }
                    // If this a fragment texture for an animated texture, modify the texture Id
                    // of the texture in the sequence
                    case OBJTEXT_TYPE_ANIMATEDTEXTURE:
                        {
                        bitu16* lpwAnimatedTextId = (bitu16*)lpFragText2->lpUserData;
                        *lpwAnimatedTextId = wIndex;
                        break;
                        }
                    }
                lpFragText2->boAlreadyDone = true; // this fragment texture is now processed
                }

            wIndex++;
            lpFragText->boAlreadyDone = false;
            }
        }

    *lpdwAreaWidth  = dwAreaWidth;
    *lpdwAreaHeight = dwAreaHeight;

    return wIndex;
}
