#pragma once

typedef enum { OBJTEXT_TYPE_POLY, OBJTEXT_TYPE_SPRITE, OBJTEXT_TYPE_SPRITE2, OBJTEXT_TYPE_ANIMATEDTEXTURE } enumOBJTEXTType;

class CFragment;

class FRAGMENT_TEXTURE
{
public:

    TR2_OBJECT_TEXTURE  ObjText;
    bool                boQuad;
    void*               lpUserData;
    enumOBJTEXTType     ObjTextType;
    bool                boAlreadyDone;

    friend bool     inline operator  != (const FRAGMENT_TEXTURE& Frag1,const FRAGMENT_TEXTURE& Frag2);
};

inline bool operator != (const FRAGMENT_TEXTURE& Frag1,const FRAGMENT_TEXTURE& Frag2)
{
    if(Frag1.boQuad != Frag2.boQuad) return true;
    if(Frag1.ObjText.wTile != Frag2.ObjText.wTile) return true;
    if(Frag1.ObjText.wAttribute != Frag2.ObjText.wAttribute) return true;
    DWORD   dwI = Frag1.boQuad ? 4 : 3;
    bool    boOk = false;
    while(dwI--)
        {
        // Warning: to speed tests up, the TR2_OBJECT_TEXTURE_VERT structure is casted
        // to a DWORD because sizeof(TR2_OBJECT_TEXTURE_VERT) = 4. In a nice world, we should test:
        // Frag1.ObjText.Vertices[dwI].bXPixel == Frag2.ObjText.Vertices[dwI].bXPixel &&
        // Frag1.ObjText.Vertices[dwI].bYPixel == Frag2.ObjText.Vertices[dwI].bYPixel && etc
        DWORD   dwData1 = * (DWORD*) (&Frag1.ObjText.Vertices[dwI]);
        DWORD   dwData2 = * (DWORD*) (&Frag2.ObjText.Vertices[dwI]);

        if(dwData1 != dwData2)
            {
            boOk = true;
            break;
            }
        }
    return boOk;
}

typedef std::list<FRAGMENT_TEXTURE> LISTOBJTEXT;

//
//
//
class CFragment
{
public:

    static void     vCollectFragments(CTRFile* lpTRFile, CFragment** lplpLstFragments,DWORD* lpdwNumFragments);
    static void     vCollectFragmentsV2(CTRFile* lpTRFile, CFragment** lplpLstFragments,DWORD* lpdwNumFragments);
    static void     vCollectFragments(CTRFile* lpTRFile, TR123_MESH* lpMesh,CFragment** lplpLstFragments,DWORD* lpdwNumFragments,DWORD dwNumMeshes = 1);
    static void     vOrganizeFragments(CTRFile* lpTRFile,DWORD dwNumFragments,CFragment* lpLstFragments,BYTE* lpbArea,
            DWORD dwAreaWidth,bool boCalculateOnly,DWORD& dwRemHeight,DWORD& dwWidthMax,bool boAlign256 = false);
    static void     vSortFragments(CFragment* lpLstFragments,int dwIdxLeft,int dwIdxRight);
    static DWORD    dwBuildTextureFromFragments(CTRFile* lpTRFile,CFragment* lpLstFragments,DWORD dwNumFragments,BYTE** lplpbArea,DWORD* lpdwAreaWidth,DWORD* lpdwAreaHeight,bool boAlign256 = false);
    static DWORD    dwRemoveDuplicated(DWORD dwNumFragments,CFragment* lpLstFragments);

public:

    static void     vCollectFragments(CTRFile* lpTRFile, TR2_OBJECT_TEXTURE* lpOText, DWORD dwNumPolys, BYTE bNumVertexPerPoly,void* lpLstPolys,CFragment* lpLstFragments,DWORD& dwNumFragments);
    static void     vCollectFragments(BYTE* lpbTextureData, DWORD dwTextureWidth, TR2_OBJECT_TEXTURE* lpOText, DWORD dwNumPolys, BYTE bNumVertexPerPoly,void* lpLstPolys,CFragment* lpLstFragments,DWORD& dwNumFragments);
    static void     vCollectFragments(CTrLstMsh* lpMsh, CFragment* lpLstFragments,DWORD& dwNumFragments);
    static void     vAddFragment(CFragment* lpLstFragments,DWORD& dwNumFragments,TR2_OBJECT_TEXTURE* lpObjVert,BYTE bNumVertexInObjText, enumOBJTEXTType objTextType,void* lpUserData,bool boDontRegroup = false,BYTE* lpbTextureData = NULL,DWORD dwTextureWidth = 256);

    LISTOBJTEXT         m_LstTextCoord;
    BYTE*               m_lpbTextureData;
    DWORD               m_dwTextureWidth;
    bit16               m_sWidth, m_sHeight;
    DWORD               m_dwArea;
    bit16               m_sUMin, m_sVMin, m_sUMax, m_sVMax;
    bit16               m_sNewUMin, m_sNewVMin;
};