#include "stdafx.h"

#include "Resource.h"
#include "MainFrm.h"
#include "FileTree.h"
#include "HandleStaticMeshes.h"
#include "HandleMesh.h"
#include "EditMesh.h"


void    CHandleMesh::vCreatePane()
{
    m_lpSplitterWnd->CreateView(0,1,RUNTIME_CLASS(CEditMesh),CSize(1,1),NULL);
}


void    CHandleMesh::vEditSetInfo(TCHAR* lpszNodeName,bool boForceNew /*= false*/)
{
    CEditMesh*  lpEdit = reinterpret_cast<CEditMesh*>(m_lpSplitterWnd->GetPane(0,1));
    if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMesh))) lpEdit->vSetDataTypeHandler(this);
}


void    CHandleMesh::vSetToolboxType()
{
    ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_MESH);
}


CMenu*  CHandleMesh::lpGetPopupupMenu(HTREEITEM hSelectedItem)
{
    m_lpMenu = new CMenu;
    if(m_boStaticMesh) m_lpMenu->LoadMenu(IDR_MENU_STATICMESH); else m_lpMenu->LoadMenu(IDR_MENU_MESH);

    return m_lpMenu->GetSubMenu(0);
}


void    CHandleMesh::vReleasePopupupMenu()
{
    delete m_lpMenu;
    
    m_lpMenu = NULL;
}


void    CHandleMesh::vSetChanged(bool boReloadTextures)
{
    if(boReloadTextures)
        {
        CEditMesh*  lpEdit = reinterpret_cast<CEditMesh*>(m_lpSplitterWnd->GetPane(0,1));
        if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMesh))) lpEdit->vReloadTextures();
        }

    CHandleBase::vSetChanged();
}


void    CHandleMesh::vHandleMenu(WORD wMenuId)
{
    switch(wMenuId)
        {
        // Delete the static mesh
        case ID_STATICMESH_DELETE:
            {
            vDelete();
            break;
            }
        // Export the mesh to a .3DS file
        case ID_SMESH_EXPORTTO3DS:
            {
            vExportTo3DS();
            break;
            }
        // Load a 3ds file and replace the current mesh
        case ID_SMESH_IMPORTFROM3DS:
            {
            vImportFrom3DS();
            break;
            }
        // Replace the current mesh with an empty mesh
        case ID_MESH_CLEAR:
        case ID_SMESH_CLEAR:
            {
            vClearMesh();
            break;
            }
        // Export the mesh to a .TRMSH / .TRSMH file
        case ID_SMESH_EXPORTASMSH:
            {
            vExportAsMSH();
            break;
            }
        // Import the mesh from a .TRMSH / .TRSMH file
        case ID_SMESH_IMPORTFROMMSH:
            {
            vImportFromMSH();
            break;
            }
        // Export the texture to a .TGA file
        case ID_SMESH_EXPORTTEXTUREASTGA:
            {
            vExportTextureToTGA();
            break;
            }
        // Import the texture from a .TGA file
        case ID_SMESH_IMPORTTEXTUREFROMTGA:
            {
            vImportTextureFromTGA();
            break;
            }
        }
}


void    CHandleMesh::vExportTo3DS()
{
    // Get the filename
    CString strFilter(_T("3DS files (*.3ds)|*.3ds||"));
    CFileDialog FileDialog(FALSE,_T("3ds"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    DWORD       dwNumMesh;
    CTrLstMsh*  lptrMSH = GetMesh(dwNumMesh);

    CString cstrFilePath = FileDialog.GetPathName();
    TCHAR*  lpszFilePath = (TCHAR*)(LPCTSTR)cstrFilePath;
    TCHAR   lpszObjName[_MAX_FNAME];
    TCHAR   szTextureName[9];

    _tsplitpath(lpszFilePath,NULL,NULL,lpszObjName,NULL);
    if(lstrlen(lpszObjName) <= 8)
        {
        lstrcpy(szTextureName, lpszObjName);
        }
    else
        {
        lstrcpy(szTextureName, &lpszObjName[lstrlen(lpszObjName) - 8]);
        }

    boExportMSHTo3DS(*lptrMSH,dwNumMesh,lpszFilePath,szTextureName);
}


void    CHandleMesh::vImportFrom3DS()
{
    BYTE* lpData = NULL;

    // Load the 3DS file
    CString strFilter(_T("3DS files (*.3ds)|*.3ds||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTRFile* lpTRFile = m_lpTree->lpGetTRFile();

    DWORD       dwNumMesh;
    CTrLstMsh*  lpMsh = GetMesh(dwNumMesh);
    TR123_MESH  MeshCopy;

    DWORD dwObjId = lpMsh->dwGetObjectId();
    
    if(lpTRFile->boImportMeshFrom3DS(&MeshCopy,(TCHAR*)(LPCTSTR)FileDialog.GetPathName(),false))
        {
        CTrLstMsh Msh(&MeshCopy,lpTRFile);

        lpMsh->vStealData(Msh);

        if(m_boStaticMesh)
            {
            TR123_MESH* lpMesh = const_cast<TR123_MESH*>(lpMsh->lpGetMesh(0));

            lpMesh->lCollisionSize = 0;
            CTRFile::vComputeMeshExtents(lpMesh);

            CTrStatMsh* lpStatMsh = static_cast<CTrStatMsh*>(lpMsh);
            CBoundingBox    BB;

            lpStatMsh->vSetObjectId(0);
            lpStatMsh->vComputeBoundingBox(BB,0);
            lpStatMsh->vSetBBVisibility(BB);
            lpStatMsh->vSetBBCollision(BB);
            lpStatMsh->vSetFlags(TRSTATMSH_FLAGS_VISIBLE);
            }

        lpMsh->vSetObjectId(dwObjId);

        vSetChanged(true);
        vEditSetInfo(m_szNodeName);
        }

    CTRFile::vReleaseMeshMemory(&MeshCopy);
}


void    CHandleMesh::vClearMesh()
{
    DWORD dwNumMesh;
    CTrLstMsh* lpMesh = GetMesh(dwNumMesh);

    lpMesh->vClearMesh(dwNumMesh);

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Save as a .TRMSH
//
//
void    CHandleMesh::vExportAsMSH()
{
    // Get the filename
    CString strFilter(_T("TR Mesh files (*.trmsh)|*.trmsh||"));
    CString strExt(_T("trmsh"));

    if(m_boStaticMesh)
        {
        strFilter = _T("TR Static Mesh files (*.trsmh)|*.trsmh||");
        strExt = _T("trsmh");
        }

    CFileDialog FileDialog(FALSE,strExt,m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    if(m_boStaticMesh)
        {
        DWORD dwNumMesh;
        CTrStatMsh*  lpMsh = static_cast<CTrStatMsh*>(GetMesh(dwNumMesh));

        lpMsh->boSave(FileDialog.GetPathName());
        }
    else
        {
        DWORD dwNumMesh;
        CTrLstMsh*  lpMsh = GetMesh(dwNumMesh);

        lpMsh->boSave(FileDialog.GetPathName());
        }
}


//
//
// Load a .TRMSH
//
//
void    CHandleMesh::vImportFromMSH()
{
    // Load the MSH file
    CString strFilter(_T("TR Mesh files (*.trmsh)|*.trmsh||"));

    if(m_boStaticMesh)
        {
        strFilter = _T("TR Static Mesh files (*.trsmh)|*.trsmh||");
        }

    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTRFile* lpTRFile = m_lpTree->lpGetTRFile();

    if(m_boStaticMesh)
        {
        DWORD dwNumMesh;
        CTrStatMsh* lpStaticMesh = static_cast<CTrStatMsh*>(GetMesh(dwNumMesh));

        DWORD dwObjId = lpStaticMesh->dwGetObjectId();

        // Build the TR static mesh
        CTrStatMsh Msh = CTrStatMsh::objLoad(FileDialog.GetPathName());

        lpStaticMesh->vStealData(Msh);
        lpStaticMesh->vSetObjectId(dwObjId);
        }
    else
        {
        DWORD dwNumMesh;
        CTrLstMsh* lpMesh = GetMesh(dwNumMesh);

        DWORD dwObjId = lpMesh->dwGetObjectId();

        // Build the TR mesh
        CTrLstMsh Msh = CTrLstMsh::objLoad(FileDialog.GetPathName());

        lpMesh->vStealData(Msh);
        lpMesh->vSetObjectId(dwObjId);
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Export the texture to a .TGA file
//
//
void    CHandleMesh::vExportTextureToTGA()
{
    // Get the filename
    CString strFilter(_T("TGA files (*.tga)|*.tga||"));
    CFileDialog FileDialog(FALSE,_T("tga"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Save the texture
    DWORD       dwNumMesh;
    CTrLstMsh*  lpMesh = GetMesh(dwNumMesh);

    lpMesh->vSaveTextureAsTGA((LPCTSTR)FileDialog.GetPathName());
}


//
//
// Import the texture from a .TGA file
//
//
void    CHandleMesh::vImportTextureFromTGA()
{
    // Load the TGA file
    CString strFilter(_T("Gfx files (*.tga;*.bmp;*.gif)|*.tga;*.bmp;*.gif|Others (*.fli;*.flc;*.cel)|*.fli;*.flc;*.cel||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Load and replace the texture
    DWORD       dwNumMesh;
    CTrLstMsh*  lpMesh = GetMesh(dwNumMesh);

    bool boOk = lpMesh->boLoadTextureFromTGA((LPCTSTR)FileDialog.GetPathName());

    if(!boOk)
        {
        CString s;
        
        s.Format(   _T("Could not replace the texture. Please, make sure that:\n\n")
                    _T("    - your graphic file is a valid one\n")
                    _T("    - your texture is %ldx%ld"),
                    lpMesh->dwGetTextureWidth(), lpMesh->dwGetTextureHeight());

        AfxMessageBox(s,MB_OK | MB_ICONSTOP);

        return;
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Delete the mesh
//
//
void    CHandleMesh::vDelete()
{
    if(AfxMessageBox(_T("Are you sure you want to delete this static mesh ?"), MB_YESNO | MB_ICONEXCLAMATION) != IDYES) return;

    DWORD       dwNumMesh;

    CTrStatMsh* lpStatMsh = static_cast<CTrStatMsh*>(GetMesh(dwNumMesh));

    CTRFile* lpTRFile = m_lpTree->lpGetTRFile();

    TRListStaticMesh* lpList = lpTRFile->GetTRFile().tr123File.lpListStaticMesh;

    for(TRListStaticMesh::iterator theIterator = lpList->begin(); theIterator != lpList->end(); theIterator++)
        {
        if(*theIterator == lpStatMsh)
            {
            lpList->erase(theIterator);

            CEditMesh*  lpEdit = reinterpret_cast<CEditMesh*>(m_lpSplitterWnd->GetPane(0,1));
            if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMesh))) lpEdit->vRelease();

            ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_EMPTY);

            delete lpStatMsh;

            vSetChanged();
            m_lpTree->m_lpHandleStaticMeshes->vExpandTree(true);
            break;
            }
        }
}
