#include "stdafx.h"

#include "Resource.h"
#include "MainFrm.h"
#include "FileTree.h"
#include "HandleMoveables.h"
#include "HandleMoveable.h"
#include "HandleMesh.h"
#include "EditMoveable.h"
#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "AnimMgr.h"
#include "TRAnimTrack.h"


void    CHandleMoveable::vCreatePane()
{
    m_lpSplitterWnd->CreateView(0,1,RUNTIME_CLASS(CEditMoveable),CSize(1,1),NULL);
}


void    CHandleMoveable::vExpandTree(bool boAlreadyExpanded)
{
    if(boAlreadyExpanded) return;

    CString             s;
    DWORD               dwI;
    HTREEITEM           h;
    CTRFile*            lpTRFile    = m_lpTree->lpGetTRFile();
    enumTRType          trType      = lpTRFile->GetTRType();

    TCHAR               szMoveableName[256+1];
    
    lstrcpyn(szMoveableName,lpszGetTitle(),256+1);

    switch(trType)
        {
        //
        case TR1_FILE:
        case TR2_FILE:
        case TR3_FILE:
            {
            CTrMvb* lpMvb = GetMoveable();
            for(dwI = 0; dwI < lpMvb->dwGetNumMeshes(); dwI++)
                {
                s.Format(_T("Mesh_%.3d"),dwI);
                INSERT(m_lpTree,h,m_hParentItem,new CHandleMesh(m_lpTree,m_lpSplitterWnd,(void*)lpMvb,false,dwI))
                }
            break;
            }
        }
}


void    CHandleMoveable::vEditSetInfo(TCHAR* lpszNodeName,bool boForceNew /*= false*/)
{
    CEditMoveable*  lpEdit = reinterpret_cast<CEditMoveable*>(m_lpSplitterWnd->GetPane(0,1));
    if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMoveable))) lpEdit->vSetDataTypeHandler(this,boForceNew);
}


void    CHandleMoveable::vSetToolboxType()
{
    ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_MOVEABLE);
}


CMenu*  CHandleMoveable::lpGetPopupupMenu(HTREEITEM hSelectedItem)
{
    m_lpMenu = new CMenu;
    m_lpMenu->LoadMenu(IDR_MENU_MOVEABLE);

    return m_lpMenu->GetSubMenu(0);
}


void    CHandleMoveable::vReleasePopupupMenu()
{
    delete m_lpMenu;
    
    m_lpMenu = NULL;
}


void    CHandleMoveable::vSetChanged(bool boReloadTextures)
{
    if(boReloadTextures)
        {
        CEditMoveable*  lpEdit = reinterpret_cast<CEditMoveable*>(m_lpSplitterWnd->GetPane(0,1));
        if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMoveable))) lpEdit->vReloadTextures();
        }

    CHandleBase::vSetChanged();
}


void    CHandleMoveable::vHandleMenu(WORD wMenuId)
{
    switch(wMenuId)
        {
        // Delete the moveable
        case ID_MOVEABLE_DELETE:
            {
            vDelete();
            break;
            }
        // Export the moveable to a .3DS file
        case ID_MOVEABLE_EXPORTTO3DS:
            {
            vExportTo3DS();
            break;
            }
        // Load a 3ds file and replace the current moveable
        case ID_MOVEABLE_IMPORTFROM3DS:
            {
            vImportFrom3DS();
            break;
            }
        // Replace the meshes of the moveable with empty meshes
        case ID_MOVEABLE_CLEAR:
            {
            vClearMoveable();
            break;
            }
        // Export the moveable to a .TRMVB file
        case ID_MOVEABLE_EXPORTASMVB:
            {
            vExportAsMVB();
            break;
            }
        // Load a trmvb file and replace the current moveable
        case ID_MOVEABLE_IMPORTFROMMVB:
            {
            vImportFromMVB();
            break;
            }
        // Export the texture to a .TGA file
        case ID_MOVEABLE_EXPORTTEXTUREASTGA:
            {
            vExportTextureToTGA();
            break;
            }
        // Import the texture from a .TGA file
        case ID_MOVEABLE_LOADREPLACETEXTURE:
            {
            vImportTextureFromTGA();
            break;
            }
        // Remove all sound references into the moveable (in the anim commands)
        case ID_MOVEABLE_REMOVESOUNDS:
            {
            vRemoveSounds();
            break;
            }
        }
}


void    CHandleMoveable::vExportTo3DS()
{
    // Get the filename
    CString strFilter(_T("3DS files (*.3ds)|*.3ds||"));
    CFileDialog FileDialog(FALSE,_T("3ds"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTrMvb*  lptrMVB = GetMoveable();

    CString cstrFilePath = FileDialog.GetPathName();
    TCHAR*  lpszFilePath = (TCHAR*)(LPCTSTR)cstrFilePath;
    TCHAR   lpszObjName[_MAX_FNAME];
    TCHAR   szTextureName[9];

    _tsplitpath(lpszFilePath,NULL,NULL,lpszObjName,NULL);
    if(lstrlen(lpszObjName) <= 8)
        {
        lstrcpy(szTextureName, lpszObjName);
        }
    else
        {
        lstrcpy(szTextureName, &lpszObjName[lstrlen(lpszObjName) - 8]);
        }

    boExportMoveableTo3DS(*lptrMVB,lpszFilePath,szTextureName,m_lpTree->lpGetTRFile());
}


void    CHandleMoveable::vImportFrom3DS()
{
    BYTE* lpData = NULL;

    // Load the 3DS file
    CString strFilter(_T("3DS files (*.3ds)|*.3ds||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTRFile* lpTRFile = m_lpTree->lpGetTRFile();

    CTrMvb* lpMoveable = GetMoveable();

    DWORD dwId = AfxMessageBox("Do you want internal lighting ? With internal lighting, lighting at each vertex is precomputed and is not affected by lights in the rooms."
            " On the contrary, with external lighting, normals at each vertex are precomputed, which allow lighting to be computed during the game according to dark/light areas."
            " \nPlease note that for the moment, with internal lighting, the precomputed lightings are all 1.0, ie full light.",MB_YESNO);
}


void    CHandleMoveable::vClearMoveable()
{
    CTrMvb*         lpMoveable = GetMoveable();

    for(DWORD dwI = 0; dwI < lpMoveable->dwGetNumMeshes(); dwI++)
        {
        lpMoveable->vClearMesh(dwI);
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Save as a .TRMVB
//
//
void    CHandleMoveable::vExportAsMVB()
{
    // Get the filename
    CString strFilter(_T("TR Moveable files (*.trmvb)|*.trmvb||"));
    CFileDialog FileDialog(FALSE,_T("trmvb"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTrMvb*  lptrMVB = GetMoveable();

    lptrMVB->boSave(FileDialog.GetPathName());
}


void    CHandleMoveable::vImportFromMVB()
{
    // Load the MVB file
    CString strFilter(_T("TR moveable files (*.trmvb)|*.trmvb||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTrMvb* lpMoveable = GetMoveable();

    DWORD dwOldObjId = lpMoveable->dwGetObjectId();

    // Build the TR moveable
    CTrMvb Mvb = CTrMvb::objLoad(FileDialog.GetPathName());

    lpMoveable->vStealData(Mvb);

    DWORD dwNewObjId = lpMoveable->dwGetObjectId();

    CTRFile*            lpTRFile = m_lpTree->lpGetTRFile();
    TR123_FILE_STRUCT&  trFile = lpTRFile->GetTRFile().tr123File;
    bool                boTR1 = lpTRFile->GetTRType() == TR1_FILE;

    // Replace the old object id in items list by the new one
    TRListItem* lpListItem = trFile.lpListItem;

    for(TRListItem::iterator theIterator4 = lpListItem->begin(); theIterator4 != lpListItem->end(); theIterator4++)
        {
        TR123_ITEM* lpItem= *theIterator4;

        if(boTR1)
            {
            if(lpItem->tr1.sObjectId == dwOldObjId) lpItem->tr1.sObjectId = dwNewObjId;
            }
        else
            {
            if(lpItem->tr2.sObjectId == dwOldObjId) lpItem->tr2.sObjectId = dwNewObjId;
            }
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName,true);
}


//
//
// Export the texture to a .TGA file
//
//
void    CHandleMoveable::vExportTextureToTGA()
{
    // Get the filename
    CString strFilter(_T("TGA files (*.tga)|*.tga||"));
    CFileDialog FileDialog(FALSE,_T("tga"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Save the texture
    CTrMvb*     lpMvb = GetMoveable();

    lpMvb->vSaveTextureAsTGA((LPCTSTR)FileDialog.GetPathName());
}


//
//
// Import the texture from a .TGA file
//
//
void    CHandleMoveable::vImportTextureFromTGA()
{
    // Load the TGA file
    CString strFilter(_T("Gfx files (*.tga;*.bmp;*.gif)|*.tga;*.bmp;*.gif|Others (*.fli;*.flc;*.cel)|*.fli;*.flc;*.cel||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Load and replace the texture
    CTrMvb*     lpMvb = GetMoveable();

    bool boOk = lpMvb->boLoadTextureFromTGA((LPCTSTR)FileDialog.GetPathName());

    if(!boOk)
        {
        CString s;
        
        s.Format(   _T("Could not replace the texture. Please, make sure that:\n\n")
                    _T("    - your graphic file is a valid one\n")
                    _T("    - your texture is %ldx%ld"),
                    lpMvb->dwGetTextureWidth(), lpMvb->dwGetTextureHeight());

        AfxMessageBox(s,MB_OK | MB_ICONSTOP);

        return;
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Remove all sound references from the anim commands
//
//
void    CHandleMoveable::vRemoveSounds()
{
    CTrMvb*     lpMvb       = GetMoveable();
    CAnimMgr*   lpAnimMgr   = lpMvb->lpGetAnimMgr();

    for(DWORD dwTrack = 0; dwTrack < lpAnimMgr->dwGetNumTracks(); dwTrack++)
        {
        CTRAnimTrack*   lpTrack     = static_cast<CTRAnimTrack*>(lpAnimMgr->lpGetTrack(dwTrack));

        DWORD dwI = lpTrack->dwGetNumAnimCommands();

        while(dwI--)
            {
            const CTRAnimCommand& AnimCmde = lpTrack->GetAnimCommand(dwI);

            bit16   sCmd = AnimCmde.sGetCommand().sValue;

            if(  sCmd == ANIMCMD_PLAYSOUNDONFRAME_2  ||  
                (sCmd == ANIMCMD_MISCACTIONONFRAME_2  && 
                    (AnimCmde.sGetParam(1).sValue == ANIMCMD_MISC_ADDITIONALSOUND1 || AnimCmde.sGetParam(1).sValue == ANIMCMD_MISC_ADDITIONALSOUND2)))
                {
                lpTrack->vRemoveAnimCommand(dwI);
                dwI = lpTrack->dwGetNumAnimCommands();
                }
            }
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Delete the moveable
//
//
void    CHandleMoveable::vDelete()
{
    if(AfxMessageBox(_T("Are you sure you want to delete this moveable ?"), MB_YESNO | MB_ICONEXCLAMATION) != IDYES) return;

    CTrMvb* lpMoveable = GetMoveable();
    DWORD   dwObjId = lpMoveable->dwGetObjectId();

    CTRFile*            lpTRFile = m_lpTree->lpGetTRFile();
    TR123_FILE_STRUCT&  trFile = lpTRFile->GetTRFile().tr123File;
    bool                boTR1 = lpTRFile->GetTRType() == TR1_FILE;

    TRListMoveable* lpList = lpTRFile->GetTRFile().tr123File.lpListMoveable;

    for(TRListMoveable::iterator theIterator = lpList->begin(); theIterator != lpList->end(); theIterator++)
        {
        if(*theIterator == lpMoveable)
            {
            // Remove from the items list
            TRListItem* lpListItem = trFile.lpListItem;

            for(TRListItem::iterator theIterator4 = lpListItem->begin(); theIterator4 != lpListItem->end(); theIterator4++)
                {
                TR123_ITEM* lpItem= *theIterator4;

                if(boTR1)
                    if(lpItem->tr1.sObjectId == dwObjId) { lpListItem->erase(theIterator4); delete lpItem; }
                else
                    if(lpItem->tr2.sObjectId == dwObjId) { lpListItem->erase(theIterator4); delete lpItem; }
                }

            //
            lpList->erase(theIterator);

            CEditMoveable*  lpEdit = reinterpret_cast<CEditMoveable*>(m_lpSplitterWnd->GetPane(0,1));
            if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMoveable))) lpEdit->vRelease();

            ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_EMPTY);

            delete lpMoveable;

            vSetChanged();
            m_lpTree->m_lpHandleMoveables->vExpandTree(true);
            break;
            }
        }
}
