#include "stdafx.h"

#include "Resource.h"
#include "MainFrm.h"
#include "FileTree.h"
#include "HandleSprites.h"
#include "HandleSprite.h"
#include "EditSprite.h"


void    CHandleSprite::vCreatePane()
{
    m_lpSplitterWnd->CreateView(0,1,RUNTIME_CLASS(CEditSprite),CSize(1,1),NULL);
}


void    CHandleSprite::vEditSetInfo(TCHAR* lpszNodeName,bool boForceNew /*= false*/)
{
    CEditSprite*  lpEdit = reinterpret_cast<CEditSprite*>(m_lpSplitterWnd->GetPane(0,1));
    if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditSprite))) lpEdit->vSetDataTypeHandler(this);
}


void    CHandleSprite::vSetToolboxType()
{
    ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_SPRITE);
}


CMenu*  CHandleSprite::lpGetPopupupMenu(HTREEITEM hSelectedItem)
{
    m_lpMenu = new CMenu;
    m_lpMenu->LoadMenu(IDR_MENU_SPRITE);

    return m_lpMenu->GetSubMenu(0);
}


void    CHandleSprite::vReleasePopupupMenu()
{
    delete m_lpMenu;
    
    m_lpMenu = NULL;
}


void    CHandleSprite::vSetChanged(bool boReloadTextures)
{
    if(boReloadTextures)
        {
        CEditSprite*  lpEdit = reinterpret_cast<CEditSprite*>(m_lpSplitterWnd->GetPane(0,1));
        if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditSprite))) lpEdit->vReloadTextures();
        }

    CHandleBase::vSetChanged();
}


void    CHandleSprite::vHandleMenu(WORD wMenuId)
{
    switch(wMenuId)
        {
        // Delete the sprite
        case ID_SPRITE_DELETE:
            {
            vDelete();
            break;
            }
        // Export the mesh to a .TRMSH file
        case ID_SPRITE_EXPORTASSPR:
            {
            vExportAsSPR();
            break;
            }
        case ID_SPRITE_IMPORTFROMSPR:
            {
            vImportFromSPR();
            break;
            }
        // Export the texture to a .TGA file
        case ID_SPRITE_EXPORTTEXTUREASTGA:
            {
            vExportTextureToTGA();
            break;
            }
        // Import the texture from a .TGA file
        case ID_SPRITE_LOADREPLACETEXTURE:
            {
            vImportTextureFromTGA();
            break;
            }
        }
}


//
//
// Save as a .TRSPR
//
//
void    CHandleSprite::vExportAsSPR()
{
    // Get the filename
    CString strFilter(_T("TR Sprite files (*.trspr)|*.trspr||"));
    CString strExt(_T("trspr"));

    CFileDialog FileDialog(FALSE,strExt,m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTrLstSpr*  lpSpr = GetSpr();

    lpSpr->boSave(FileDialog.GetPathName());
}


//
//
// Load a .TRSPR
//
//
void    CHandleSprite::vImportFromSPR()
{
    // Load the SPR file
    CString strFilter(_T("TR Sprite files (*.trspr)|*.trspr||"));

    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    CTrLstSpr* lpSpr = GetSpr();

    // Build the TR sprite sequence
    CTrLstSpr Spr = CTrLstSpr::objLoad(FileDialog.GetPathName());

    lpSpr->vStealData(Spr);

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Export the texture to a .TGA file
//
//
void    CHandleSprite::vExportTextureToTGA()
{
    // Get the filename
    CString strFilter(_T("TGA files (*.tga)|*.tga||"));
    CFileDialog FileDialog(FALSE,_T("tga"),m_szNodeName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Save the texture
    CTrLstSpr*     lpSpr = GetSpr();

    lpSpr->vSaveTextureAsTGA((LPCTSTR)FileDialog.GetPathName());
}


//
//
// Import the texture from a .TGA file
//
//
void    CHandleSprite::vImportTextureFromTGA()
{
    // Load the TGA file
    CString strFilter(_T("Gfx files (*.tga;*.bmp;*.gif)|*.tga;*.bmp;*.gif|Others (*.fli;*.flc;*.cel)|*.fli;*.flc;*.cel||"));
    CFileDialog FileDialog(TRUE,_T(""),_T(""),OFN_HIDEREADONLY,strFilter);

    if(FileDialog.DoModal() != IDOK) return;

    CWaitCursor dummy;

    // Load and replace the texture
    CTrLstSpr*  lpSpr = GetSpr();

    bool boOk = lpSpr->boLoadTextureFromTGA((LPCTSTR)FileDialog.GetPathName());

    if(!boOk)
        {
        CString s;
        
        s.Format(   _T("Could not replace the texture. Please, make sure that:\n\n")
                    _T("    - your graphic file is a valid one\n")
                    _T("    - your texture is %ldx%ld"),
                    lpSpr->dwGetTextureWidth(), lpSpr->dwGetTextureHeight());

        AfxMessageBox(s,MB_OK | MB_ICONSTOP);

        return;
        }

    vSetChanged(true);

    vEditSetInfo(m_szNodeName);
}


//
//
// Delete the sprite
//
//
void    CHandleSprite::vDelete()
{
    if(AfxMessageBox(_T("Are you sure you want to delete this sprite ?"), MB_YESNO | MB_ICONEXCLAMATION) != IDYES) return;

    CTrLstSpr*  lpSpr = GetSpr();
    DWORD       dwObjId = lpSpr->dwGetObjectId();

    CTRFile*            lpTRFile = m_lpTree->lpGetTRFile();
    TR123_FILE_STRUCT&  trFile = lpTRFile->GetTRFile().tr123File;
    bool                boTR1 = lpTRFile->GetTRType() == TR1_FILE;

    TRListSpriteSeq* lpList = lpTRFile->GetTRFile().tr123File.lpListSpriteSeq;

    for(TRListSpriteSeq::iterator theIterator = lpList->begin(); theIterator != lpList->end(); theIterator++)
        {
        if(*theIterator == lpSpr)
            {
            // Remove from the items list
            TRListItem* lpListItem = trFile.lpListItem;

            for(TRListItem::iterator theIterator4 = lpListItem->begin(); theIterator4 != lpListItem->end(); theIterator4++)
                {
                TR123_ITEM* lpItem= *theIterator4;

                if(boTR1)
                    if(lpItem->tr1.sObjectId == dwObjId) { lpListItem->erase(theIterator4); delete lpItem; }
                else
                    if(lpItem->tr2.sObjectId == dwObjId) { lpListItem->erase(theIterator4); delete lpItem; }
                }

            //
            lpList->erase(theIterator);

            CEditSprite*  lpEdit = reinterpret_cast<CEditSprite*>(m_lpSplitterWnd->GetPane(0,1));
            if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditSprite))) lpEdit->vRelease();

            ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_EMPTY);

            delete lpSpr;

            vSetChanged();
            m_lpTree->m_lpHandleSprites->vExpandTree(true);
            break;
            }
        }
}
