// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "TRViewer.h"

#include "MainFrm.h"
#include "TRViewerDoc.h"
#include "TRViewerView.h"
#include "Toolbox.h"
#include "ToolboxEmpty.h"
#include "ToolboxMesh.h"
#include "ToolboxMoveable.h"
#include "ToolboxSprite.h"
#include "Debug.h"

#ifdef _DEBUG
//!#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TOOLBOX_TOPMOST

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_UPDATE_COMMAND_UI(ID_VIEW_OBJECTTOOLBOX, OnUpdateViewObjecttoolbox)
	ON_COMMAND(ID_VIEW_OBJECTTOOLBOX, OnViewObjecttoolbox)
	ON_COMMAND(ID_MISC_DEBUG, OnMiscDebug)
	ON_UPDATE_COMMAND_UI(ID_RENDERING_MANUALCAMERA, OnUpdateRenderingManualcamera)
	ON_COMMAND(ID_RENDERING_MANUALCAMERA, OnRenderingManualcamera)
	ON_UPDATE_COMMAND_UI(ID_RENDERING_SHOWAXIS, OnUpdateRenderingShowAxis)
	ON_COMMAND(ID_RENDERING_SHOWAXIS, OnRenderingShowaxis)
	ON_COMMAND(ID_RENDERING_CHOOSEBKGCOLOR, OnRenderingChoosebkgcolor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	m_boToolboxVisible = false;
    m_ToolboxType = TOOLBOX_EMPTY;
    
    m_LstToolbox = new CToolbox* [TOOLBOX_NUM];
    m_LstToolbox[ TOOLBOX_EMPTY ] = new CToolboxEmpty;
    m_LstToolbox[ TOOLBOX_EMPTY ]->vInit(CToolboxEmpty::IDD, &m_boToolboxVisible);
    m_LstToolbox[ TOOLBOX_MESH ] = new CToolboxMesh;
    m_LstToolbox[ TOOLBOX_MESH ]->vInit(CToolboxMesh::IDD, &m_boToolboxVisible);
    m_LstToolbox[ TOOLBOX_MOVEABLE ] = new CToolboxMoveable;
    m_LstToolbox[ TOOLBOX_MOVEABLE ]->vInit(CToolboxMoveable::IDD, &m_boToolboxVisible);
    m_LstToolbox[ TOOLBOX_SPRITE ] = new CToolboxSprite;
    m_LstToolbox[ TOOLBOX_SPRITE ]->vInit(CToolboxSprite::IDD, &m_boToolboxVisible);

	m_boManualCamera = false;
    m_boShowAxis = false;
    m_BkgColor = RGB(192,192,192);
}

CMainFrame::~CMainFrame()
{
    for(DWORD dwI = 0; dwI < TOOLBOX_NUM; dwI++) delete m_LstToolbox[ dwI ];

    delete [] m_LstToolbox;
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(12) | TBBS_CHECKBOX;
    if(m_boToolboxVisible) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(12, iStyle);

    iStyle = m_wndToolBar.GetButtonStyle(13) | TBBS_CHECKBOX;
    if(m_boManualCamera) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(13, iStyle);

    iStyle = m_wndToolBar.GetButtonStyle(14) | TBBS_CHECKBOX;
    if(m_boShowAxis) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(14, iStyle);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CMDIFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnUpdateViewObjecttoolbox(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boToolboxVisible ? 1 : 0);
}


void CMainFrame::OnViewObjecttoolbox() 
{
    vSetToolboxType(m_ToolboxType,true,!m_boToolboxVisible);
}


void CMainFrame::OnUpdateRenderingManualcamera(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boManualCamera ? 1 : 0);
}


void CMainFrame::OnRenderingManualcamera() 
{
    vSetCameraMode(!m_boManualCamera);
}


void CMainFrame::OnUpdateRenderingShowAxis(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boShowAxis? 1 : 0);
}


void CMainFrame::OnRenderingShowaxis() 
{
	vSetShowAxis(!m_boShowAxis);
}


void CMainFrame::vSetCameraMode(bool boManual)
{
    m_boManualCamera = boManual;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(13);
    if(m_boManualCamera) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(13, iStyle);

    CTRViewerView*  lpView = static_cast<CTRViewerView*>(CTRViewerView::GetActiveView());

    if(!lpView) return;

    lpView->vSetCameraMode(boManual);
}


void CMainFrame::vSetShowAxis(bool boShow)
{
    m_boShowAxis = boShow;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(14);
    if(m_boShowAxis) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(14, iStyle);

    CTRViewerView*  lpView = static_cast<CTRViewerView*>(CTRViewerView::GetActiveView());

    if(!lpView) return;

    lpView->vSetShowAxis(boShow);
}


void CMainFrame::OnMiscDebug()
{
    vMiscDebug();
}


void CMainFrame::vSetToolboxType(enumToolboxType ToolboxType,bool boSetVisibility,bool boShow)
{
    DWORD dwX = -1, dwY = -1;
    RECT rect;

    if(!boSetVisibility) boShow = m_boToolboxVisible;

    if(ToolboxType != m_ToolboxType)
        {
        if(m_boToolboxVisible)
            {
            m_LstToolbox[(int) m_ToolboxType]->ShowWindow(SW_HIDE);
            m_LstToolbox[(int) m_ToolboxType]->GetWindowRect(&rect);
            dwX = rect.left;
            dwY = rect.top;
            }
        }

    m_ToolboxType = ToolboxType;

    if(dwX != -1  &&  dwY != -1)
        {
        DWORD dwW, dwH;

        m_LstToolbox[(int) m_ToolboxType]->GetWindowRect(&rect);
        dwW = rect.right - rect.left;
        dwH = rect.bottom - rect.top;
        m_LstToolbox[(int) m_ToolboxType]->SetWindowPos(NULL,dwX,dwY,dwW,dwH,0);
        }

#ifndef TOOLBOX_TOPMOST
    m_LstToolbox[(int) m_ToolboxType]->SetWindowPos(0,0,0,0,0,SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
#else
    m_LstToolbox[(int) m_ToolboxType]->SetWindowPos(&wndTopMost,0,0,0,0,SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
#endif
    m_LstToolbox[(int) m_ToolboxType]->ShowWindow(boShow ? SW_SHOW : SW_HIDE);

    m_boToolboxVisible = boShow;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(12);
    if(m_boToolboxVisible) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(12, iStyle);
}


CToolbox* CMainFrame::lpGetToolbox()
{
    return m_LstToolbox[(int) m_ToolboxType];
}


CToolbox* CMainFrame::lpGetToolbox(enumToolboxType ToolboxType)
{
    return m_LstToolbox[(int) ToolboxType];
}

void CMainFrame::OnRenderingChoosebkgcolor() 
{
    CColorDialog    dlg(m_BkgColor,CC_FULLOPEN,this);
    
    if(dlg.DoModal() == IDOK)
        {
        m_BkgColor = dlg.GetColor();

        CTRViewerView*  lpView = static_cast<CTRViewerView*>(CTRViewerView::GetActiveView());

        if(!lpView) return;

        lpView->vSetBkgColor(m_BkgColor);
        }
}
