// OpenGLWnd1.cpp : implementation file
//

#include "stdafx.h"
#include <math.h>

#include "TRViewer.h"
#include "MainFrm.h"
#include "OpenGLWnd1.h"
#include "TextureTools.h"
#include "ToolboxMoveable.h"

#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "AnimMgr.h"

#ifdef _DEBUG
//!#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern bool g_boSavingLevel;

/////////////////////////////////////////////////////////////////////////////
// COpenGLWnd

COpenGLWnd::COpenGLWnd()
{
    m_hRC = NULL;
    m_hDC = NULL;
    m_boMakeCurrentOk = false;
    m_LstTextId = NULL;

    CMainFrame* lpMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);

    // General / rendering parameters
    m_boAutomaticCamera     = !lpMainFrame->boIsManualCamera();
    m_boShowAxis            = lpMainFrame->boShowAxis();
    m_BkgColor              = lpMainFrame->colGetBkgColor();

    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
    m_fFrameRate        = 1.0f;

    m_boPickingMode = false;
    m_dwShowNumVertex = -1;         // in picking mode, num vertex to select

    m_lpToolbox = reinterpret_cast<CToolboxMoveable*> (lpMainFrame->lpGetToolbox(TOOLBOX_MOVEABLE));

    m_boRotWithMouse = false;
    m_boTransWithMouse = false;

    m_lpTrGfx               = NULL;
    m_trObjType             = TrObj_Type_Mesh;

    // Mesh / static mesh parameters
    m_boShowCollSphere      = false;
    m_boShowVertices        = false;
    m_boShowStaticBB1       = false;
    m_boStaticBB1IsSel      = false;
    m_boShowStaticBB2       = false;
    m_boStaticBB2IsSel      = false;
    m_dwNumMeshToRender     = 0;

    // Anim parameters
    m_boAutomaticAnim       = true;
    m_boAnimShow            = false;
    m_boChainAnimations     = false;
    m_boAnimInterp          = true;
    m_boRecalcInterp        = false;
    m_dwAnimCurAnim         = 0;
    m_fAnimOverallSpeed     = (FLOAT) 1.0;

    // Sprite parameters
    m_fSprSeqSpeed          = 20.0f;
    m_boSprSeqShowBorder    = false;
    m_boSprSeqTypeAnimated  = true;
    m_fSprSeqListWidth      = 256.0f;
    m_lpSprSeq              = NULL;
}


COpenGLWnd::~COpenGLWnd()
{
    m_lpTrGfx = NULL;

    vReleaseTextures();

    if(m_hRC)
        {
        //wglMakeCurrent( NULL, NULL );
        wglDeleteContext(m_hRC);
        }

    m_hRC = NULL;
}


void COpenGLWnd::vReleaseTextures()
{
    if(m_LstTextId)
        {
        glDeleteTextures(1,m_LstTextId);
        delete [] m_LstTextId;
        }

    m_LstTextId = NULL;
}


BEGIN_MESSAGE_MAP(COpenGLWnd, CWnd)
	//{{AFX_MSG_MAP(COpenGLWnd)
	ON_WM_PAINT()
	ON_WM_SIZE()
    ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// COpenGLWnd message handlers

void COpenGLWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
    vDraw((HDC)dc);

	// Do not call CFrameWnd::OnPaint() for painting messages
}


int COpenGLWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_hDC = ::GetDC(m_hWnd);			// Get the Device context

    static PIXELFORMATDESCRIPTOR pfd =
    {
        sizeof(PIXELFORMATDESCRIPTOR),          // Size of this structure
        1,                                      // Version of this structure    
        0
        | PFD_SUPPORT_OPENGL                    // The buffer supports OpenGL 
        | PFD_GENERIC_ACCELERATED               // The pixel format is 
        | PFD_DOUBLEBUFFER                      // The buffer is 
        | PFD_SWAP_COPY                         // Specifies the content of 
        | PFD_SWAP_EXCHANGE                     // Specifies the content of 
        ,
        PFD_TYPE_RGBA,                          // RGBA Color mode
        16,                                     // Want 16bit color 
        0,0,0,0,0,0,                            // Not used to select mode
        0,0,                                    // Not used to select mode
        0,0,0,0,0,                              // Not used to select mode
        16,                                     // Size of depth buffer
        0,                                      // Not used to select mode or ignored
        0,                                      //  idem
        PFD_MAIN_PLANE ,                        // Draw in main plane
        0,                                      // Not used to select mode or ignored
        0,0,0                                   //  idem
    };

    int nMyPixelFormatID = ChoosePixelFormat( m_hDC, &pfd );

    if(!nMyPixelFormatID)
        {
        CString s;

        s.Format("Error when calling OpenGL ChoosePixelFormat function (%ld)",GetLastError());
        AfxMessageBox(s,MB_OK | MB_ICONSTOP);
        }
    else
        {
        SetPixelFormat( m_hDC, nMyPixelFormatID, &pfd );

        m_hRC = wglCreateContext( m_hDC );

        //wglMakeCurrent(m_hDC, m_hRC);
        m_boMakeCurrentOk = true;

        RECT rect;
        
        GetClientRect(&rect);        
        
        vChangeSize(rect.right - rect.left,rect.bottom - rect.top);
        }

	return 0;
}


void COpenGLWnd::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
    vChangeSize(cx,cy);
}


void COpenGLWnd::vChangeSize(int cx, int cy)
{
    if(!m_boMakeCurrentOk) return;

    GLsizei     glnWidth, glnHeight;

    glnWidth    = (GLsizei) cx;
    glnHeight   = (GLsizei) cy;
    m_gldAspect = (GLdouble)glnWidth / (GLdouble)glnHeight;

    wglMakeCurrent(m_hDC, m_hRC);

    glViewport( 0, 0, glnWidth, glnHeight );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluPerspective(
        60.0,           // Field-of-view angle
        m_gldAspect,    // Aspect ratio of view volume
        0.1,            // Distance to near clipping plane
        10.0 );         // Distance to far clipping plane

    glMatrixMode( GL_MODELVIEW );

    wglMakeCurrent(NULL, NULL);
}


void COpenGLWnd::vInitiateCamera(float fMinX,float fMaxX,float fMinY,float fMaxY,float fMinZ,float fMaxZ, bool boCenterCamera)
{
    // Center of the bounding box
    float x = (fMaxX + fMinX) / 2.0f;
    float y = (fMaxY + fMinY) / 2.0f;
    float z = (fMaxZ + fMinZ) / 2.0f;

    // Calculate the largest side of the bounding box
    float fZRange = fMaxZ - fMinZ;
    float fYRange = fMaxY - fMinY;
    float fXRange = fMaxX - fMinX;

    float fMax = fZRange;
    
    if(fMax < fYRange) fMax = fYRange;
    if(fMax < fXRange) fMax = fXRange;

    // Save 1/fMax for later use (scaling of the objects before the rendering, so that it fits in a box of 1 unit side)
    m_fMax = 1 / fMax;

    // x,y,z = coord. of the lookat -> the camera will focus at this point
    x = x * m_fMax;
    y = y * m_fMax;
    z = z * m_fMax;

    // Calculate the camera's position (we know the mesh fits in a box of 1 unit side now)
    float fEyeX = -(1.0f - 1.0f);
    float fEyeY = -(-1.0f + 0.3f);
    float fEyeZ = 1.0f + 1.0f;

    float fLookAtX = x;
    float fLookAtY = y;
    float fLookAtZ = z;

    if(boCenterCamera)
        {
        fEyeX = x;
        fEyeY = y;
        fEyeZ = 2.0f;

        fLookAtX = x;
        fLookAtY = y;
        fLookAtZ = z;
        }

    // Set the parameters of the camera
    m_Camera.CamSetPosition(vec3(fEyeX,fEyeY,fEyeZ));
    m_Camera.CamSetLookAt(vec3(fLookAtX,fLookAtY,fLookAtZ));
    m_Camera.CamCalcViewUp(0);
    m_Camera.CamCalcMatrix();

    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
    m_fFrameRate        = 1.0f;
}


void    COpenGLWnd::vSetAutomaticCamera(bool boOk)
{
    m_boAutomaticCamera = boOk;
    
    if(boOk)
        {
        m_dwOldTime         = -1;
        m_OldTime.QuadPart  = 0;
        }
}


void    COpenGLWnd::vSetShowAxis(bool boOk)
{
    m_boShowAxis = boOk;
}


void    COpenGLWnd::vSetBkgColor(COLORREF BkgColor)
{
    m_BkgColor = BkgColor;
}


void COpenGLWnd::vGenerateTextures()
{
    // Generate textures for OpenGL
    if(!m_LstTextId  &&  (m_lpTrGfx  ||  m_lpSprSeq))
        {
        m_LstTextId = new GLuint[1];

        glPixelStorei(GL_UNPACK_ALIGNMENT,1);
        glGenTextures(1,m_LstTextId);

        glBindTexture(GL_TEXTURE_2D, m_LstTextId[0]);
        //glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
        //glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        
        CTrObjGfx* lpGfx = NULL;
        
        if(m_lpTrGfx) lpGfx = m_lpTrGfx; else lpGfx = m_lpSprSeq;

        DWORD dwWidth   = 256;
        DWORD dwHeight  = 256;

        if(lpGfx->dwGetTextureWidth()  > dwWidth)  dwWidth  = lpGfx->dwGetTextureWidth();
        if(lpGfx->dwGetTextureHeight() > dwHeight) dwHeight = lpGfx->dwGetTextureHeight();

        BYTE* lpData = new BYTE[dwWidth * dwHeight * 4];

        ASSERT(lpGfx != NULL);

        vPromoteTo32Bits(const_cast<BYTE*>(lpGfx->lpGetTextureData()),dwWidth,dwHeight,lpGfx->dwGetTextureDepth(),NULL,lpData,FALSE,lpGfx->dwGetTextureWidth(),lpGfx->dwGetTextureHeight());

        glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,dwWidth,dwHeight,0,GL_BGRA_EXT,GL_UNSIGNED_BYTE,lpData);

        delete [] lpData;

        m_dwOldTime         = -1;
        m_OldTime.QuadPart  = 0;
        }
}


void COpenGLWnd::vActivateWnd()
{
    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
}


void COpenGLWnd::vDraw(HDC hDC)
{
    if(!m_boMakeCurrentOk  ||  (!m_lpTrGfx  &&  !m_lpSprSeq)  ||  g_boSavingLevel) return;

    // Calculate the framerate
    bool  boUpdateCamera = (m_dwOldTime != -1)  &&  m_boAutomaticCamera;

    DWORD dwNewTime = GetTickCount();
    DWORD dwElapsedTime = dwNewTime - m_dwOldTime;

    if(!dwElapsedTime) dwElapsedTime = 1;

    m_fFrameRate = 1000.0f / dwElapsedTime;

    m_dwOldTime = dwNewTime;

    // Calculate a more accurate framerate (if supported by the OS)
    LARGE_INTEGER NewTime;
    LARGE_INTEGER Frequency;

    if(QueryPerformanceCounter(&NewTime)  &&  QueryPerformanceFrequency(&Frequency))
        {
        LONGLONG llElapsedTime = NewTime.QuadPart - m_OldTime.QuadPart;

        m_fFrameRate = (float) ((double)Frequency.QuadPart / (double)llElapsedTime);

        if(m_fFrameRate < 0) m_fFrameRate = 20.0f;

        m_OldTime = NewTime;
        }

    // Update the camera orientation (if automatic mode)
    if(boUpdateCamera)
        {
        if(m_fFrameRate < 5.0f) m_fFrameRate = 30.0f;
        m_Camera.CamRotateAxis(360.0f / (3 * m_fFrameRate));
        m_Camera.CamCalcViewUp(0);
        m_Camera.CamCalcMatrix();
        }

    // Set OpenGL parameters for the rendering
    wglMakeCurrent(m_hDC, m_hRC);

    glClearColor(GetRValue(m_BkgColor) / 255.0f, GetGValue(m_BkgColor) / 255.0f, GetBValue(m_BkgColor) / 255.0f,0x00);

    glClearDepth(1.f);
    glShadeModel(GL_SMOOTH);
    glEnable( GL_DEPTH_TEST );
    glDepthFunc(GL_LESS);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_CULL_FACE);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
    glAlphaFunc(GL_GREATER, 0.5);

    vGenerateTextures();

    // Push the camera matrix transformation on the stack
    vec3 vec3Eye    = m_Camera.CamGetPosition();
    vec3 vec3LookAt = m_Camera.CamGetLookAt();
    vec3 vec3ViewUp = m_Camera.CamGetViewUp();

    glLoadIdentity();
    gluLookAt(vec3Eye[0],vec3Eye[1],vec3Eye[2],vec3LookAt[0],vec3LookAt[1],vec3LookAt[2],-vec3ViewUp[0],-vec3ViewUp[1],-vec3ViewUp[2]);

    // Transformation in model space (a scale to fit the geometry in a box of 1 unit side)
    glScalef(m_fMax,m_fMax,m_fMax);

    // Render the geometry
    if(m_lpTrGfx)
        {
        switch(m_trObjType)
            {
            case TrObj_Type_Moveable:
                vRenderMoveable(static_cast<CTrMvb*>(m_lpTrGfx));
                break;
            default:
                vRenderMesh(m_lpTrGfx,m_dwNumMeshToRender,m_boShowCollSphere,false,m_boShowVertices);
                break;
            }
        }
    else if(m_lpSprSeq) vRenderSprSeq(m_lpSprSeq);

    // Animate the moveable
    if(m_boAnimShow  &&  m_trObjType == TrObj_Type_Moveable)
        {
        if(m_boAutomaticAnim  ||  m_boRecalcInterp)
            {
            if(CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack())
                {
                lpAnim->vActivateInterpolation(m_boAnimInterp);

                m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
                m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
                m_lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());

                if(m_boAutomaticAnim)
                    {
                    if(!static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->boRunForward(1.0f / m_fFrameRate))
                        {
                        CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();
                        DWORD       dwNextTrackIndex, dwNextTrackFrame;

                        m_dwAnimCurAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->dwGetCurrentTrack();
                        m_lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
                        m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
                        m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
                        m_lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
                        static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
                        m_lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
                        }
                    }

                m_boRecalcInterp = false;
                }
            }
        }

    // Display the framerate
    TCHAR szFrameRate[32];

    wsprintf(szFrameRate,"%.4d fps",(DWORD)m_fFrameRate);

    //TextOut(hDC,2,2,szFrameRate,lstrlen(szFrameRate));

    SwapBuffers(hDC);

    //TextOut(hDC,2,2,szFrameRate,lstrlen(szFrameRate));

    wglMakeCurrent(NULL, NULL);
}


void COpenGLWnd::vPick(DWORD dwPosX,DWORD dwPosY)
{
    #define BUFSIZE     512

    GLuint  selectBuf[BUFSIZE];
    GLint   hits;
    GLint   viewport[4];

    m_boPickingMode = true;

    wglMakeCurrent(m_hDC, m_hRC);

    glGetIntegerv(GL_VIEWPORT, viewport);
    glSelectBuffer(BUFSIZE, selectBuf);
    glRenderMode(GL_SELECT);

    glInitNames();
    glPushName(0);

    glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    gluPickMatrix((GLdouble)dwPosX,(GLdouble)(viewport[3] - dwPosY),8.0,8.0,viewport);
    gluPerspective(
        60.0,           // Field-of-view angle
        m_gldAspect,    // Aspect ratio of view volume
        0.1,            // Distance to near clipping plane
        10.0 );         // Distance to far clipping plane

    // Push the camera matrix transformation on the stack
    vec3 vec3Eye    = m_Camera.CamGetPosition();
    vec3 vec3LookAt = m_Camera.CamGetLookAt();
    vec3 vec3ViewUp = m_Camera.CamGetViewUp();

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt(vec3Eye[0],vec3Eye[1],vec3Eye[2],vec3LookAt[0],vec3LookAt[1],vec3LookAt[2],-vec3ViewUp[0],-vec3ViewUp[1],-vec3ViewUp[2]);

    // Transformation in model space (a scale to fit the geometry in a box of 1 unit side)
    glScalef(m_fMax,m_fMax,m_fMax);

    // Render the geometry
    if(m_lpTrGfx  &&  m_dwNumMeshToRender != -1) vRenderMeshVertices(m_lpTrGfx->lpGetMesh(m_dwNumMeshToRender));

    glFlush();
    hits = glRenderMode(GL_RENDER);
    glMatrixMode( GL_PROJECTION );
    glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
    m_boPickingMode = false;
    wglMakeCurrent(NULL, NULL);

    DWORD dwMinZ = 0xFFFFFFFF;
    m_dwShowNumVertex = -1;

    while(hits--)
        {
        if(selectBuf[4*hits+1] < dwMinZ)
            {
            dwMinZ = selectBuf[4*hits+1];
            m_dwShowNumVertex = selectBuf[4*hits+3]-1;
            }
        }

    InvalidateRect(NULL);
}


void COpenGLWnd::vCheckKeys()
{
    bool boCapslocked   = GetAsyncKeyState(VK_CAPITAL) & 0x8001 ? true : false;

    if(!boCapslocked) return;

    bit16 sXSgn = GetAsyncKeyState(VK_LEFT)  & 0x8001 ? -1 : GetAsyncKeyState(VK_RIGHT) & 0x8001 ? +1 : 0;
    bit16 sYSgn = GetAsyncKeyState(VK_PRIOR) & 0x8001 ? +1 : GetAsyncKeyState(VK_NEXT)  & 0x8001 ? -1 : 0;
    bit16 sZSgn = GetAsyncKeyState(VK_UP)    & 0x8001 ? -1 : GetAsyncKeyState(VK_DOWN)  & 0x8001 ? +1 : 0;
    bit16 sWSgn = GetAsyncKeyState(VK_END)   & 0x8001 ? -1 : GetAsyncKeyState(VK_DELETE)& 0x8001 ? +1 : 0;

    if(!sXSgn  &&  !sYSgn  &&  !sZSgn  &&  !sWSgn) return;

    m_Camera.CamMoveForward(sYSgn * 0.05f * 100.0f/m_fFrameRate);
    m_Camera.CamRotateAxis(sXSgn * 100.0f / m_fFrameRate);
    m_Camera.CamRotate(sZSgn * 100.0f / m_fFrameRate);
    m_Camera.CamRoll(sWSgn * 100.0f / m_fFrameRate);

    m_Camera.CamCalcMatrix();
}


void COpenGLWnd::vSetGfxAddr(CTrLstMsh* lpTrMsh,enumTrObjType trObjType,bool boDontReleaseTextures,DWORD dwNumMeshToRender)
{
    if(m_lpTrGfx  &&  !boDontReleaseTextures)
        {
        vReleaseTextures();
        }

    m_lpTrGfx               = lpTrMsh;
    m_trObjType             = trObjType;
    m_dwNumMeshToRender     = dwNumMeshToRender;

    m_dwShowNumVertex       = -1;
    m_boAnimShow            = false;

    //
    // Calculate the initial position / lookat of the camera. We calculate a coeff to apply to each coord. of the
    // mesh's vertices so that the mesh fits in a box of 1 unit side. So, in calculating the camera's position/lookat,
    // we assume the mesh is in this box
    //

    CBoundingBox BoundingBox;

    lpTrMsh->vComputeBoundingBox(BoundingBox,m_dwNumMeshToRender);

    if(BoundingBox.GetBBLo()[0] >= BoundingBox.GetBBHi()[0])
        {
        // Handle the case where all meshes are dummy mesh
        // (if so, there's no valid bounding box, so create a default one)
        BoundingBox.vSetBB(CVect3D(-50,-50,-50),CVect3D(+50,+50,+50));
        }

    const CVect3D& v1 = BoundingBox.GetBBLo();
    const CVect3D& v2 = BoundingBox.GetBBHi();

    vInitiateCamera(v1[0],v2[0],v1[1],v2[1],v1[2],v2[2]);

    InvalidateRect(NULL,FALSE);

    if(m_trObjType == TrObj_Type_Moveable)
        {
        static_cast<CTrMvb*>(lpTrMsh)->lpGetAnimMgr()->vSetChainTracks(m_boChainAnimations);
        static_cast<CTrMvb*>(lpTrMsh)->lpGetAnimMgr()->vSetOverallSpeed(m_fAnimOverallSpeed);
        }
}


void COpenGLWnd::vSetSprSeqAddr(CTrLstSpr* lpSprSeq,bool boDontReleaseTextures)
{
    if(m_lpSprSeq  &&  !boDontReleaseTextures)
        {
        vReleaseTextures();
        }

    m_lpSprSeq      = lpSprSeq;

    m_dwShowNumVertex   = -1;
    m_boAnimShow        = false;
    m_boAutomaticCamera = false;

    CMainFrame* lpMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);

    lpMainFrame->vSetCameraMode(!m_boAutomaticCamera);

    //
    // Calculate the initial position / lookat of the camera. We calculate a coeff to apply to each coord. of the
    // mesh's vertices so that the mesh fits in a box of 1 unit side. So, in calculating the camera's position/lookat,
    // we assume the mesh is in this box
    //

    // Get the extents of the bounding box for the mesh
    const WORLD_POSITIONNING* lpWorldP = lpSprSeq->lpGetWorldPositionning();

    float fMinX = lpWorldP[0].sLeftSide, fMaxX = lpWorldP[0].sRightSide;
    float fMinY = -lpWorldP[0].sBottomSide, fMaxY = -lpWorldP[0].sTopSide;
    float fMinZ = -200, fMaxZ = 200;

    if(fMinX < fMaxX)
        {
        vInitiateCamera(fMinX,fMaxX,fMinY,fMaxY,fMinZ,fMaxZ,true);

        InvalidateRect(NULL,FALSE);
        }
}


void COpenGLWnd::vModifyIntensOfSelectedVertices(bit16 sStep)
{
    if(!sStep  ||  m_dwShowNumVertex == -1  ||  !m_lpTrGfx  ||  m_dwNumMeshToRender == -1) return;
    
    TR123_MESH* lpMesh = const_cast<TR123_MESH*>(m_lpTrGfx->lpGetMesh(m_dwNumMeshToRender));

    if(lpMesh->Lighting.LightingType != MESH_LIGHTING_INTERNAL) return;

    bit16 sIntens = lpMesh->Lighting.lpLights[m_dwShowNumVertex];

    sIntens += sStep;
    if(sIntens >= 8192) sIntens = 8192;

    lpMesh->Lighting.lpLights[m_dwShowNumVertex] = sIntens;
}


void    COpenGLWnd::vSetMoveableAnimation(DWORD dwNumAnim)
{
    if(m_trObjType != TrObj_Type_Moveable) return;

    if(dwNumAnim == -1)
        {
        m_boAnimShow = false;
        return;
        }

    m_boAnimShow = true;
    m_dwAnimCurAnim = dwNumAnim;
    static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vSetCurrentTrack(m_dwAnimCurAnim);

    CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();
    DWORD       dwNextTrackIndex,dwNextTrackFrame;

    m_lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
    m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    m_lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
    static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
    m_lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
}


void    COpenGLWnd::vSetAnimationType(bool boChainAnimations,bool boResetMoveable)
{
    if(m_trObjType != TrObj_Type_Moveable) return;

    m_boChainAnimations = boChainAnimations;
    static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vSetChainTracks(boChainAnimations);

    if(boResetMoveable) vSetMoveableAnimation(m_dwAnimCurAnim);
}


void    COpenGLWnd::vSetAnimOverallSpeed(float fSpeed)
{
    if(m_trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

    FLOAT m_fAnimOverallSpeed;

    if(fSpeed >= 0.5f)
        m_fAnimOverallSpeed = (FLOAT) ((fSpeed - 0.5f) * 5.0f + 1.0f);
    else
        m_fAnimOverallSpeed = (FLOAT) (fSpeed * 2.0f);

    static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vSetOverallSpeed(m_fAnimOverallSpeed);
}


void    COpenGLWnd::vSetAnimCurFrame(DWORD dwNumFrame)
{
    if(m_trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

    CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

    lpAnim->boSetCurrentFrame(dwNumFrame);

    m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    m_lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());
}


void    COpenGLWnd::vSetAnimCurKey(DWORD dwNumKey)
{
    if(m_trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

    CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

    lpAnim->boSetCurrentFrame(dwNumKey * lpAnim->dwGetFrameRate());

    m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    m_lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());
}


void    COpenGLWnd::vSetAnimInterp(bool boOk)
{
    m_boAnimInterp = boOk;
}


void    COpenGLWnd::vChangeAnimInterp(int iDir)
{
    if(m_trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

    CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

    float fFrame = lpAnim->fGetCurFrame();

    fFrame += iDir * ((FLOAT)1.0 / (FLOAT)30.0) * lpAnim->fGetOverallSpeed();

    if(!static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->boSetCurrentFrame(fFrame))
        {
        DWORD dwNextTrackIndex,dwNextTrackFrame;

        lpAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

        m_dwAnimCurAnim = static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->dwGetCurrentTrack();
        m_lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
        m_lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
        static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
        m_lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
        }

    m_lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    m_lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());

    m_boRecalcInterp = true;
}


void    COpenGLWnd::vSetAnimInterp(float fInterp)
{
    if(m_trObjType == TrObj_Type_Moveable  &&  static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) static_cast<CTrMvb*>(m_lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()->vSetInterpolationFactor(fInterp);
}


void COpenGLWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_boRotWithMouse = true;
    m_RotMousePos = point;
}

void COpenGLWnd::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_boRotWithMouse = false;
}

void COpenGLWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
	m_boTransWithMouse = true;
    m_TransMousePos = point;
}

void COpenGLWnd::OnRButtonUp(UINT nFlags, CPoint point) 
{
	m_boTransWithMouse = false;
}

void COpenGLWnd::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
    if(m_boAutomaticCamera) return;

    m_Camera.CamMoveForward(zDelta * 0.2f / m_fFrameRate);
    m_Camera.CamCalcMatrix();
}

void COpenGLWnd::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    if(m_boAutomaticCamera) return;

    if(m_lpTrGfx)           vSetGfxAddr(m_lpTrGfx,m_trObjType,true,m_dwNumMeshToRender);
    else if(m_lpSprSeq)     vSetSprSeqAddr(m_lpSprSeq,true);
}

void COpenGLWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
    if(m_boAutomaticCamera) return;

	if(m_boRotWithMouse)
        {
        int iDeltaX = point.x - m_RotMousePos.x;
        int iDeltaY = point.y - m_RotMousePos.y;

        float fXSgn = -iDeltaX * 2 * 100.0f / m_fFrameRate;
        float fZSgn = -iDeltaY * 2 * 100.0f / m_fFrameRate;

        m_Camera.CamRotateAxis(fXSgn);
        m_Camera.CamRotate(fZSgn);
        m_Camera.CamCalcMatrix();

        m_RotMousePos = point;
        }

    if(m_boTransWithMouse)
        {
        int iDeltaX = point.x - m_TransMousePos.x;
        int iDeltaY = point.y - m_TransMousePos.y;

        float fXSgn = -iDeltaX * 0.01f * 100.0f / m_fFrameRate;
        float fYSgn = -iDeltaY * 0.01f * 100.0f / m_fFrameRate;

        m_Camera.CamMoveLeft(fXSgn);
        m_Camera.CamMoveUp(fYSgn);
        m_Camera.CamCalcMatrix();

        m_TransMousePos = point;
        }
}
