#include "stdafx.h"

#include "Resource.h"
#include "3DSStuff.h"
#include "TGAFile.h"

#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "AnimMgr.h"


bool    boExportMoveableTo3DS(CTrMvb& trMVB,TCHAR* lpsz3DSName,TCHAR* lpszTextureName,CTRFile* lpTRFile)
{
    // Extract the name of the moveable from the .3DS path
    TCHAR lpszObjName[_MAX_FNAME];
    TCHAR szDrive[_MAX_DRIVE];
    TCHAR szDir[_MAX_DIR];

    _tsplitpath(lpsz3DSName,szDrive,szDir,lpszObjName,NULL);

    // Construct the path to the texture file
    TCHAR szTexturePath[MAX_PATH];
    CString s;

    _tmakepath(szTexturePath,szDrive,szDir,lpszTextureName,_T(".tga"));

    // Init the 3DS database
    database3ds* Db3ds = NULL;

    Init3DSForSave(&Db3ds);

    float fMinX = 1e20f, fMaxX = -1e20f;
    float fMinY = 1e20f, fMaxY = -1e20f;
    float fMinZ = 1e20f, fMaxZ = -1e20f;

    // Create the material for the moveable texture
    CREATE_MATERIAL_FOR_TILE()

    // Copy the meshes into the 3DS database
    bool boOk = true;

    #define STACK_DEPTHMAX 10
    DWORD   Stack[STACK_DEPTHMAX];
    float   StackTrans[3 * STACK_DEPTHMAX];
    float   fTransX = 0, fTransY = 0, fTransZ = 0;
    DWORD   dwStackPointer = 0;

    TR2_MESHTREE* lpMeshTree = trMVB.m_lpMeshTree;
    DWORD   dwParent = 0;

    DWORD           dwNumMeshes     = trMVB.m_dwNumMeshes;

    for(DWORD dwI = 0; dwI < dwNumMeshes; dwI++)
        {
        wsprintf(lpszObjName,__TEXT("M%.3d"),dwI);
        kfmesh3ds* objMotion = NULL;

        InitObjectMotion3ds(&objMotion,1,1,1,0,0);

        sprintf(objMotion->name,"M%.3d",dwI);

        if(dwI)
            {
            bitu32  dwStack = lpMeshTree->lCoord; lpMeshTree++;
            bit32   lTransX = lpMeshTree->lCoord; lpMeshTree++;
            bit32   lTransY = lpMeshTree->lCoord; lpMeshTree++;
            bit32   lTransZ = lpMeshTree->lCoord; lpMeshTree++;

            if(dwStack & 1)
                {
                dwStackPointer--;
                dwParent = Stack[dwStackPointer];
                fTransX = StackTrans[dwStackPointer*3 + 0];
                fTransY = StackTrans[dwStackPointer*3 + 1];
                fTransZ = StackTrans[dwStackPointer*3 + 2];
                }

            if(dwStack & 2)
                {
                Stack[dwStackPointer] = dwParent;
                StackTrans[dwStackPointer*3 + 0] = fTransX;
                StackTrans[dwStackPointer*3 + 1] = fTransY;
                StackTrans[dwStackPointer*3 + 2] = fTransZ;
                dwStackPointer++;
                }

            fTransX +=  (float)lTransX;
            fTransY +=  (float)lTransY;
            fTransZ +=  (float)lTransZ;

            if(dwParent != -1) sprintf(objMotion->parent,"M%.3d",dwParent);

            objMotion->pos[0].x = (float)lTransX;
            objMotion->pos[0].y = (float)lTransZ;
            objMotion->pos[0].z = (float)-lTransY;
            }

        PutObjectMotion3ds(Db3ds,objMotion);
        ReleaseObjectMotion3ds(&objMotion);

        boOk &= boCopyTRMeshInto3DSMesh(Db3ds,lpszObjName,trMVB,&trMVB.m_lpLstMeshes[dwI],lpszTextureName,fMinX,fMaxX,fMinY,fMaxY,fMinZ,fMaxZ,fTransX,fTransY,fTransZ);

        dwParent = dwI;
        }

    if(boOk)
        {
        // Lights
        light3ds*       lpSlite = NULL;

        PUT_LIGHT(fMinX,fMinY,fMaxZ,0.5f,"Light1")
        PUT_LIGHT(fMaxX,fMinY,fMaxZ,0.5f,"Light2")
        PUT_LIGHT(fMaxX,fMaxY,fMaxZ,0.5f,"Light3")
        PUT_LIGHT(fMinX,fMaxY,fMaxZ,0.5f,"Light4")

        // Texture
        if(trMVB.m_dwTextureDepth == 16)
            vMakeTGA16Bits(szTexturePath,trMVB.m_lpbTextureData,trMVB.m_dwTextureWidth,trMVB.m_dwTextureHeight,2,true);
        else
            vMakeTGA32Bits(szTexturePath,trMVB.m_lpbTextureData,trMVB.m_dwTextureWidth,trMVB.m_dwTextureHeight,4,true);
        }

    Save3DS(&Db3ds,lpsz3DSName);

    return true;
}
