// TRViewerView.cpp : implementation of the CTRViewerView class
//

#include "stdafx.h"
#include "TRViewer.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "TRViewerDoc.h"
#include "TRViewerView.h"
#include "FileTree.h"
#include "HandleMoveables.h"
#include "HandleStaticMeshes.h"
#include "HandleSprites.h"
#include "EditMesh.h"
#include "EditMoveable.h"
#include "EditSprite.h"

#ifdef _DEBUG
//!#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTRViewerView

IMPLEMENT_DYNCREATE(CTRViewerView, CFormView)

BEGIN_MESSAGE_MAP(CTRViewerView, CFormView)
	//{{AFX_MSG_MAP(CTRViewerView)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTRViewerView construction/destruction

CTRViewerView::CTRViewerView()
	: CFormView(CTRViewerView::IDD)
{
	//{{AFX_DATA_INIT(CTRViewerView)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// TODO: add construction code here
    m_lpTree = NULL;
}

CTRViewerView::~CTRViewerView()
{
    if(m_lpTree)
        {
        delete m_lpTree;
        }

    m_lpTree = NULL;
}

void CTRViewerView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTRViewerView)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BOOL CTRViewerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CTRViewerView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	ResizeParentToFit();

    // Resize the view
    CTRViewerDoc* lpDoc = GetDocument();
    CChildFrame*  lpFrame = (CChildFrame*)GetParentFrame();

    lpFrame->MoveWindow(0,0,610,400);

    // Create the tree control
    m_lpTree = new CFileTree(lpFrame,&lpDoc->GetTRFile(),lpDoc);

    RECT Rect;

    lpFrame->GetClientRect(&Rect);

    m_lpTree->Create(WS_VISIBLE | WS_TABSTOP | WS_CHILD | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES/*   | TVS_DISABLEDRAGDROP*/,
        CRect(0, 0, Rect.right-200, Rect.bottom), this, IDC_FILETREE);

    lpFrame->m_SplitterWnd.SetColumnInfo(0,200,1);
    lpFrame->m_SplitterWnd.RecalcLayout();

    m_lpTree->vInitialize();
}


CView*  CTRViewerView::GetActiveView()
{
    CMDIChildWnd * pChild = ((CMDIFrameWnd*)(AfxGetApp()->m_pMainWnd))->MDIGetActive();
    
    if(!pChild) return NULL;
    
    CView* pView = pChild->GetActiveView();
    if(!pView) return NULL;

    //if(!pView->IsKindOf(RUNTIME_CLASS(CTRViewerView))) return NULL;
    
    return pView;
}


bool    CTRViewerView::boSave(LPCTSTR lpszFileName)
{
    return m_lpTree->boSave(lpszFileName);
}

/////////////////////////////////////////////////////////////////////////////
// CTRViewerView printing

BOOL CTRViewerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTRViewerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTRViewerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CTRViewerView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// CTRViewerView diagnostics

#ifdef _DEBUG
void CTRViewerView::AssertValid() const
{
	CFormView::AssertValid();
}

void CTRViewerView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CTRViewerDoc* CTRViewerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTRViewerDoc)));
	return (CTRViewerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTRViewerView message handlers

void CTRViewerView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);
    
    SetScaleToFitSize(CSize(cx,cy));

    if(m_lpTree) m_lpTree->SetWindowPos(NULL,0,0,cx,cy,0);	
}


BOOL CTRViewerView::boOnIdle()
{
    BOOL boMore = FALSE;

    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();
    CEditMesh*  lpEdit = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditMoveable*  lpEdit2 = reinterpret_cast<CEditMoveable*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditSprite*  lpSprite = reinterpret_cast<CEditSprite*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        boMore = lpEdit->boOnIdle();
        }
    else if(lpEdit2->IsKindOf(RUNTIME_CLASS(CEditMoveable)))
        {
        boMore = lpEdit2->boOnIdle();
        }
    else if(lpSprite->IsKindOf(RUNTIME_CLASS(CEditSprite)))
        {
        boMore = lpSprite->boOnIdle();
        }

    return boMore;
}

void CTRViewerView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();
    CEditMesh*  lpEdit = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpEdit->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        lpEdit->OnActivateView(false,NULL,NULL);
        }
	
	CFormView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}


void    CTRViewerView::vSetCameraMode(bool boManual)
{
    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();

    CEditMesh*      lpMsh       = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditMoveable*  lpMvb       = reinterpret_cast<CEditMoveable*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpMsh->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        lpMsh->vSetAutomaticCamera(!boManual);
        }
    else if(lpMvb->IsKindOf(RUNTIME_CLASS(CEditMoveable)))
        {
        lpMvb->vSetAutomaticCamera(!boManual);
        }
}


void    CTRViewerView::vSetShowAxis(bool boShow)
{
    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();

    CEditMesh*      lpMsh       = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditMoveable*  lpMvb       = reinterpret_cast<CEditMoveable*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpMsh->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        lpMsh->vSetShowAxis(boShow);
        }
    else if(lpMvb->IsKindOf(RUNTIME_CLASS(CEditMoveable)))
        {
        lpMvb->vSetShowAxis(boShow);
        }
}


void    CTRViewerView::vSetBkgColor(COLORREF BkgColor)
{
    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();

    CEditMesh*      lpMsh       = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditMoveable*  lpMvb       = reinterpret_cast<CEditMoveable*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditSprite*    lpSprite    = reinterpret_cast<CEditSprite*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpMsh->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        lpMsh->vSetBkgColor(BkgColor);
        }
    else if(lpMvb->IsKindOf(RUNTIME_CLASS(CEditMoveable)))
        {
        lpMvb->vSetBkgColor(BkgColor);
        }
    else if(lpSprite->IsKindOf(RUNTIME_CLASS(CEditSprite)))
        {
        lpSprite->vSetBkgColor(BkgColor);
        }
}


void    CTRViewerView::vResetRightView(bool boReexpandTree)
{
    CChildFrame* lpFrame = (CChildFrame*)GetParentFrame();

    CEditMesh*      lpMsh       = reinterpret_cast<CEditMesh*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditMoveable*  lpMvb       = reinterpret_cast<CEditMoveable*>(lpFrame->m_SplitterWnd.GetPane(0,1));
    CEditSprite*    lpSprite    = reinterpret_cast<CEditSprite*>(lpFrame->m_SplitterWnd.GetPane(0,1));

    if(lpMsh->IsKindOf(RUNTIME_CLASS(CEditMesh)))
        {
        lpMsh->vRelease();
        }
    else if(lpMvb->IsKindOf(RUNTIME_CLASS(CEditMoveable)))
        {
        lpMvb->vRelease();
        }
    else if(lpSprite->IsKindOf(RUNTIME_CLASS(CEditSprite)))
        {
        lpSprite->vRelease();
        }

    ((CMainFrame*)AfxGetApp()->m_pMainWnd)->vSetToolboxType(TOOLBOX_EMPTY);

    if(boReexpandTree)
        {
        if(m_lpTree->m_lpHandleMoveables    &&  m_lpTree->m_lpHandleMoveables->boIsExpanded())       m_lpTree->m_lpHandleMoveables->vExpandTree(true);
        if(m_lpTree->m_lpHandleStaticMeshes &&  m_lpTree->m_lpHandleStaticMeshes->boIsExpanded())    m_lpTree->m_lpHandleStaticMeshes->vExpandTree(true);
        if(m_lpTree->m_lpHandleSprites      &&  m_lpTree->m_lpHandleSprites->boIsExpanded())         m_lpTree->m_lpHandleSprites->vExpandTree(true);
        }
}


