#include "stdafx.h"

#include "Fragment.h"
#include "TextureTools.h"
#include "TGAFile.h"


//----------------------------------------------------------------------------
//
//  Function:   vInitialize
//
//  Synopsis:   Reset the members of the class
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrObjGfx::vInitialize()
{
    m_dwObjectId        = 0;
    m_dwNumFragText     = 0;
    m_lpFragText        = NULL;
    m_dwTextureWidth    = 0;
    m_dwTextureHeight   = 0;
    m_dwTextureDepth    = 0;
    m_lpbTextureData    = NULL;
}


//----------------------------------------------------------------------------
//
//  Function:   vRelease
//
//  Synopsis:   Release memory / Reset the members of the class
//
//  Arguments:  [boReleaseParent] -- true to release the parent class
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrObjGfx::vRelease(bool boReleaseParent /*= true*/)
{
    delete [] m_lpFragText;
    delete [] m_lpbTextureData;

    vInitialize();

    if(boReleaseParent) CTrObjBase::vRelease(true);
}


//----------------------------------------------------------------------------
//
//  Function:   vStealData
//
//  Synopsis:   Transfer data from an object to the current one: after the
//              operation, the source object has lost his data member !
//
//  Arguments:  [obj] -- object to get the data from
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrObjGfx::vStealData(CTrObjGfx& obj)
{
    CTrObjBase::vStealData(obj);

    m_dwObjectId        = obj.m_dwObjectId;
    m_dwNumFragText     = obj.m_dwNumFragText;
    m_lpFragText        = obj.m_lpFragText;
    m_dwTextureWidth    = obj.m_dwTextureWidth;
    m_dwTextureHeight   = obj.m_dwTextureHeight;
    m_dwTextureDepth    = obj.m_dwTextureDepth;
    m_lpbTextureData    = obj.m_lpbTextureData;

    obj.vInitialize();
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrObjGfx
//
//  Synopsis:   Construct a CTrObjGfx from a block in memory
//
//  Arguments:  [lpObj] -- memory block
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrObjGfx::CTrObjGfx(BYTE* lpObj) : CTrObjBase(lpObj)
{
    vInitialize();

    if(lpObj)
        {
        lpObj += CTrObjBase::dwGetDataSize();

        vConstruct(lpObj);
        }
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrObjGfx
//
//  Synopsis:   Construct a CTrObjGfx from a TR list of animated textures
//
//  Arguments:  [lpAnimatedText] -- TR list of animated textures
//              [dwDesiredTextureWidth] -- desired width of the texture
//                (0 if no matter)
//              [dwDesiredTextureHeight] -- desired height of the texture
//                (0 if no matter)
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrObjGfx::CTrObjGfx(CTRFile* lpTRFile, bitu16* lpAnimatedText, DWORD dwDesiredTextureWidth/* = 0*/,DWORD dwDesiredTextureHeight/* = 0*/) :
        CTrObjBase(0,TrObj_Type_AnimTexture,(enumTrVersion)lpTRFile->GetTRType())
{
    vInitialize();

    bitu16  wNumTextures = *lpAnimatedText++;   wNumTextures++;
    DWORD   dwNumFragments = wNumTextures;

    CFragment*  lpLstFragments = new CFragment [ dwNumFragments ];

    TR_FILE& trFile = lpTRFile->GetTRFile();

    dwNumFragments = 0;

    while(wNumTextures--)
        {
        bitu16  wNumObjText = *lpAnimatedText;
        TR2_OBJECT_TEXTURE* lpObjVert = &trFile.tr123File.lpObjectTextures[wNumObjText];

        bool    boTri = (lpObjVert->Vertices[3].bXPixel == 0)  &&
                        (lpObjVert->Vertices[3].bYPixel == 0)  &&
                        (lpObjVert->Vertices[3].cXCoordinate == 0)  &&
                        (lpObjVert->Vertices[3].cYCoordinate == 0);

        BYTE*   lpbTextureData = (BYTE*) trFile.tr123File.lplpTextiles16[lpObjVert->wTile]->awTile;

        CFragment::vAddFragment(lpLstFragments,dwNumFragments,lpObjVert, boTri ? 3 : 4, OBJTEXT_TYPE_ANIMATEDTEXTURE,(void*)lpAnimatedText, false, lpbTextureData);
        lpAnimatedText++;
        }

    // Construct CTrObjGfx
    vConstruct(lpTRFile,lpLstFragments,dwNumFragments,dwDesiredTextureWidth,dwDesiredTextureHeight);

    delete [] lpLstFragments;
}


//----------------------------------------------------------------------------
//
//  Function:   dwGetDataSize
//
//  Synopsis:   Get the size of the CTrObjGfx when put in a continuous block
//              of memory
//
//  Arguments:  -
//
//  Return:     size
//
//----------------------------------------------------------------------------
/*virtual*/ DWORD   CTrObjGfx::dwGetDataSize()
{
    DWORD   dwSize = 
                CTrObjBase::dwGetDataSize() +
                sizeof(DWORD) +                                                 /* m_dwObjectId */
                sizeof(DWORD) +                                                 /* m_dwNumFragText */
                sizeof(TR2_OBJECT_TEXTURE) * m_dwNumFragText +                  /* fragments data */
                sizeof(DWORD) +                                                 /* m_dwTextureWidth */
                sizeof(DWORD) +                                                 /* m_dwTextureHeight */
                sizeof(DWORD) +                                                 /* m_dwTextureDepth */
                m_dwTextureWidth * m_dwTextureHeight * (m_dwTextureDepth / 8);  /* texture data */

    return dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   lpbGetData
//
//  Synopsis:   Convert a CTrObjGfx to a continuous block of memory
//
//  Arguments:  [dwDataSize] -- on return, hold the size of the block
//
//  Return:     pointer to the memory block
//
//  Caution:    you must make a delete [] on the pointer returned when you
//              have finished working with it !
//----------------------------------------------------------------------------
/*virtual*/ BYTE*   CTrObjGfx::lpbGetData(DWORD& dwDataSize)
{
    BYTE*   lpbData = CTrObjBase::lpbGetData(dwDataSize);
    DWORD   dwTrObjBaseDataSize = CTrObjBase::dwGetDataSize();
    BYTE*   lpbDataSave = lpbData;

    lpbData += dwTrObjBaseDataSize;

    // Object Id
    *(DWORD*)lpbData = m_dwObjectId;    lpbData += sizeof(DWORD);

    // Fragments texture
    *(DWORD*)lpbData = m_dwNumFragText; lpbData += sizeof(DWORD);
    memcpy(lpbData, m_lpFragText, sizeof(TR2_OBJECT_TEXTURE) * m_dwNumFragText);
    lpbData += sizeof(TR2_OBJECT_TEXTURE) * m_dwNumFragText;

    // Texture
    *(DWORD*)lpbData = m_dwTextureWidth;  lpbData += sizeof(DWORD);
    *(DWORD*)lpbData = m_dwTextureHeight; lpbData += sizeof(DWORD);
    *(DWORD*)lpbData = m_dwTextureDepth;  lpbData += sizeof(DWORD);

    DWORD   dwTextureDataSize = m_dwTextureWidth * m_dwTextureHeight * (m_dwTextureDepth / 8);

    memcpy(lpbData, m_lpbTextureData, dwTextureDataSize);

    lpbData += dwTextureDataSize;

    return lpbDataSave;
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrObjGfx from a block in memory
//
//  Arguments:  [lpObjGfx] -- memory block
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrObjGfx::vConstruct(BYTE* lpObjGfx)
{
    BYTE* lpbData = lpObjGfx;

    // Object Id
    m_dwObjectId = *(DWORD*)lpbData;    lpbData += sizeof(DWORD);

    // Fragments texture
    m_dwNumFragText = *(DWORD*)lpbData; lpbData += sizeof(DWORD);
    m_lpFragText = new TR2_OBJECT_TEXTURE [ m_dwNumFragText ];
    memcpy(m_lpFragText, lpbData, sizeof(TR2_OBJECT_TEXTURE) * m_dwNumFragText);
    lpbData += sizeof(TR2_OBJECT_TEXTURE) * m_dwNumFragText;

    // Texture
    m_dwTextureWidth    = *(DWORD*)lpbData; lpbData += sizeof(DWORD);
    m_dwTextureHeight   = *(DWORD*)lpbData; lpbData += sizeof(DWORD);
    m_dwTextureDepth    = *(DWORD*)lpbData; lpbData += sizeof(DWORD);

    DWORD   dwDataSize = m_dwTextureWidth * m_dwTextureHeight * (m_dwTextureDepth / 8);

    m_lpbTextureData    = new BYTE [ dwDataSize ];

    memcpy(m_lpbTextureData, lpbData, dwDataSize);
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrObjGfx from a list of fragments
//
//  Arguments:  [lpLstFragments] -- fragments list
//              [dwNumFragments] -- number of fragments in the list
//              [dwDesiredTextureWidth] -- desired width of the texture
//                (0 if no matter)
//              [dwDesiredTextureHeight] -- desired height of the texture
//                (0 if no matter)
//
//  Return:     -
//
//  Caution:    the lpLstFragments is modified (sorted) !
//----------------------------------------------------------------------------
void    CTrObjGfx::vConstruct(CTRFile* lpTRFile, CFragment* lpLstFragments, DWORD dwNumFragments, DWORD dwDesiredTextureWidth/* = 0*/,DWORD dwDesiredTextureHeight/* = 0*/)
{
    // Sort the fragments by height, then width
    if(dwNumFragments > 1) CFragment::vSortFragments(lpLstFragments,0,dwNumFragments-1);

    // Create the texture
    BYTE*   lpbArea         = NULL;
    DWORD   dwAreaWidth     = dwDesiredTextureWidth;
    DWORD   dwAreaHeight    = dwDesiredTextureHeight;
    DWORD   dwTextureDepth  = 16;

    CFragment::dwBuildTextureFromFragments(lpTRFile,lpLstFragments,dwNumFragments,&lpbArea,&dwAreaWidth,&dwAreaHeight);

    //
    // Set the class members
    //

    // Fragments texture
    DWORD dwNumObjText = 0;

    for(DWORD dwI = 0; dwI < dwNumFragments; dwI++)
        {
        for(LISTOBJTEXT::iterator theIter = lpLstFragments[dwI].m_LstTextCoord.begin(); theIter != lpLstFragments[dwI].m_LstTextCoord.end(); theIter++)
            {
            FRAGMENT_TEXTURE* lpFragText = &(*theIter);

            if(!lpFragText->boAlreadyDone) dwNumObjText++;
            }
        }

    m_dwNumFragText = dwNumObjText;
    m_lpFragText = new TR2_OBJECT_TEXTURE [ m_dwNumFragText ];

    dwNumObjText = 0;
    for(dwI = 0; dwI < dwNumFragments; dwI++)
        {
        for(LISTOBJTEXT::iterator theIter = lpLstFragments[dwI].m_LstTextCoord.begin(); theIter != lpLstFragments[dwI].m_LstTextCoord.end(); theIter++)
            {
            FRAGMENT_TEXTURE*   lpFragText = &(*theIter);

            if(lpFragText->boAlreadyDone) continue;

            m_lpFragText[ dwNumObjText++ ] = lpFragText->ObjText;
            }
        }

    // Texture
    DWORD   dwDataSize  = dwAreaWidth * dwAreaHeight * (dwTextureDepth / 8);

    m_dwTextureWidth    = dwAreaWidth;
    m_dwTextureHeight   = dwAreaHeight;
    m_dwTextureDepth    = dwTextureDepth;
    m_lpbTextureData    = new BYTE [ dwDataSize ];

    memcpy(m_lpbTextureData, lpbArea, dwDataSize);

    delete [] lpbArea;
}


//----------------------------------------------------------------------------
//
//  Function:   vSaveTextureAsTGA
//
//  Synopsis:   Save the internal texture as a 16 or 32 bits .TGA file
//
//  Arguments:  [lpszFilename] -- name of the .TGA file
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrObjGfx::vSaveTextureAsTGA(const TCHAR* lpszFilename)
{
    if(m_dwTextureDepth == 32)
        vMakeTGA32Bits(lpszFilename,m_lpbTextureData,m_dwTextureWidth,m_dwTextureHeight,m_dwTextureDepth / 8,true);
    else
        vMakeTGA16Bits(lpszFilename,m_lpbTextureData,m_dwTextureWidth,m_dwTextureHeight,m_dwTextureDepth / 8,true);
}


//----------------------------------------------------------------------------
//
//  Function:   boLoadTextureFromTGA
//
//  Synopsis:   Load the texture from a .TGA file
//
//  Arguments:  [lpszFilename] -- name of the .TGA file
//
//  Return:     true if the texture could be replaced by the one in the .TGA
//              file, else false
//
//  Caution:    the texture must be the same size as the replaced one !
//
//----------------------------------------------------------------------------
bool    CTrObjGfx::boLoadTextureFromTGA(const TCHAR* lpszFilename)
{
    ASSERT(m_lpbTextureData != NULL  &&  m_dwTextureWidth != 0  &&  m_dwTextureHeight != 0);

    if(m_lpbTextureData == NULL  ||  m_dwTextureWidth == 0  ||  m_dwTextureHeight == 0) return false;

    DWORD   dwWidth, dwHeight;
    BYTE*   lpbData;

    DWORD   dwError = dwLoadImage(lpszFilename,&dwWidth,&dwHeight,&lpbData);

    if(dwError != 0) return false;

    if(dwWidth != m_dwTextureWidth  ||  dwHeight != m_dwTextureHeight) return false;

    delete [] m_lpbTextureData;

    vConvertABGRToARGB(m_dwTextureWidth,m_dwTextureHeight,true,lpbData,lpbData);

    m_lpbTextureData    = lpbData;
    m_dwTextureDepth    = 32;

    return true;
}
